#!/bin/sh
# **********************************************************************
#
# Copyright (c) 2005-2006
# IONA Technologies PLC
# Dublin, Ireland and Waltham, MA, USA
#
# All Rights Reserved
#
# Licensed to apache under Apache 2.0 License - 2006
# **********************************************************************

#
# Function to set "java.endorsed.dirs" property for JDK 1.5 
#
set_java_endorsed_dirs()
{
    JAVA="$JAVA -Djava.endorsed.dirs=$top_srcdir/lib"
}

#
# Messages for the non-availability of Java and set the
# clients and servers for auto testing.
#
set_auto_values_and_msgs()
{
    auto_clients="1"
    auto_servers="2 3 4 5"
}

#
# Function to set the client executable command
#
set_client()
{
    num=$1;
    cpp_dir=$2;
    java_pkg=$3;

    #
    # If a separate java package is not specified, create one based on
    # $cpp_dir
    #
    if test -z "$java_pkg"
    then
        java_pkg="test.$cpp_dir"
    fi

    if test "$num" = "1"
    then
	client="$JAVA $java_pkg.Client -ORBthreaded"
    fi
}

#
# Function to set the server executable command
#
set_server()
{
    num=$1;
    cpp_dir=$2;
    java_pkg=$3;

    #
    # If a separate java package is not specified, create one based on
    # $cpp_dir
    #
    if test -z "$java_pkg"
    then
        java_pkg="test.$cpp_dir"
    fi

    if test "$num" = "2"
    then
	server="$JAVA $java_pkg.Server -OAthreaded"
    fi

    if test "$num" = "3"
    then
	server="$JAVA $java_pkg.Server -OAthread_per_client"
    fi

    if test "$num" = "4"
    then
	server="$JAVA $java_pkg.Server -OAthread_per_request"
    fi

    if test "$num" = "5"
    then
	server="$JAVA $java_pkg.Server -OAthread_pool 10"
    fi
    server="$server -IIOPhost localhost"
}

#
# Function to set the collocated server executable command
#
set_colloc()
{
    num=$1;
    cpp_dir=$2;
    java_pkg=$3;

    #
    # If a separate java package is not specified, create one based on
    # $cpp_dir
    #
    if test -z "$java_pkg"
    then
        java_pkg="test.$cpp_dir"
    fi

    if test "$server_num" = "2"
    then
	colloc="$JAVA $java_pkg.Collocated -OAthreaded"
    fi

    if test "$server_num" = "3"
    then
	colloc="$JAVA $java_pkg.Collocated -OAthread_per_client"
    fi

    if test "$server_num" = "4"
    then
	colloc="$JAVA $java_pkg.Collocated -OAthread_per_request"
    fi

    if test "$server_num" = "5"
    then
	colloc="$JAVA $java_pkg.Collocated -OAthread_pool 10"
    fi
}

#
# Delete old log files
#
delete_logfiles()
{
    type=$1

    rm -f "$type"_colloc.log
    rm -f "$type"_client.log
    rm -f "$type"_server.log
}

#
# Function to run the test executables.
#
execute_test()
{
    #
    # The type of test being preformed
    #
    type=$1

    #
    # Set name of reference file
    #
    ref=$2

    #
    # Display info
    #
    $echo
    $echo Java CLASSPATH is \"CLASSPATH=$CLASSPATH\"
    $echo Server command is \"$server\"
    $echo Client command is \"$client\"

    #
    # Run remote tests
    #
    $echo
    $echo "Running remote tests:"

    #
    # Start server
    #
    $echo
    $echo "Waiting for server to start up... \c"
    rm -f $ref
    ( $server & echo $! > srvid ) 2>&1 | tee -a "$type"_server.log &
    count=0
    while test ! -s $ref -a $count -lt 16
    do
	sleep 1
	count=`expr $count + 1`
    done
    if test ! -s $ref
    then
	$echo "Failed!"
	$echo "(Server was not started)"
	exit
    else
	$echo "Done!"
    fi

    #
    # Start client
    #
    $client 2>&1 | tee -a "$type"_client.log

    #
    # Wait for server deactivation
    #
    $echo "Waiting for server to deactivate... \c"
    count=0
    while test -r $ref -a $count -lt 3
    do
	sleep 1
	count=`expr $count + 1`
    done

    if test -r $ref
    then
	$echo "Failed!"
	$echo "(Server was not deactivated by client - deactivating server now)"
	deactivate
    else
	$echo "Done!"
    fi
}


#
# Function to run the collocated test executables.
#
execute_colloc_test()
{
    type=$1

    $echo Collocated command is \"$colloc\"

    #
    # Run collocated tests
    #
    $echo
    $echo "Running collocated tests:"
    $echo
    $colloc 2>&1 | tee -a "$type"_colloc.log
}
