// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

public abstract class Client
{
    private int usage_; // The usage counter
    private com.ooc.OB.CodeConverters codeConverters_; // The code converters
    
    //
    // The concurrency model for this Client
    //
    final public static int Blocking = 0;
    final public static int Threaded = 2;
    protected int concModel_;

    // ----------------------------------------------------------------------
    // Client private and protected member implementations
    // ----------------------------------------------------------------------

    // ----------------------------------------------------------------------
    // Client package member implementations
    // ----------------------------------------------------------------------

    Client(int concModel, CodeConverters conv)
    {
        usage_ = 0;
        codeConverters_ = conv;
	concModel_ = concModel;
    }

    // ----------------------------------------------------------------------
    // Client public member implementations
    // ----------------------------------------------------------------------

    //
    // Destroy the client
    //
    public abstract void
    destroy(boolean terminate);

    //
    // Increment usage (not mutex protected)
    //
    public final void
    incUsage()
    {
	Assert._OB_assert(usage_ >= 0);
	usage_++;
    }

    //
    // Decrement usage (not mutex protected)
    //
    // Returns true if after the decrement the usage counter is larger
    // than 0, and false otherwise.
    //
    public final boolean
    decUsage()
    {
	Assert._OB_assert(usage_ > 0);
	usage_--;
	return usage_ > 0;
    }

    //
    // Get the codeset converters
    //
    public com.ooc.OB.CodeConverters
    codeConverters()
    {
	return codeConverters_;
    }

    //
    // Get a new request ID
    //
    public abstract int
    requestId();

    //
    // get a list of ServiceContexts that have to be sent on an AMI router
    // request
    //
    public abstract org.omg.IOP.ServiceContext[] 
    getAMIRouterSCL();
    
    //
    // Get all profiles that are usable with this client
    //
    public abstract com.ooc.OCI.ProfileInfo[]
    getUsableProfiles(org.omg.IOP.IOR ior, org.omg.CORBA.Policy[] pl);

    //
    // Get the OCI connector info
    //
    public abstract com.ooc.OCI.ConnectorInfo
    connectorInfo();

    //
    // Get the OCI transport info
    //
    public abstract com.ooc.OCI.TransportInfo
    transportInfo();

    //
    // Start a downcall, returning a downcall emitter and an
    // OutputStream for marshalling a request
    //
    public abstract DowncallEmitter
    startDowncall(Downcall down, com.ooc.CORBA.OutputStreamHolder out);

    //
    // Checks whether this client is equal to another client
    //
    public abstract boolean
    equal(Client clt);

    //
    // Force connection establishment
    //
    public abstract void
    bind(int connectTimeout);

    //
    // Determines whether this client supports twoway invocations
    //
    public abstract boolean
    twoway();
}
