// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

final public class CoreTraceLevels
{
    private int traceConnections_;
    private int traceRetry_;
    private int traceRequestsIn_;
    private int traceRequestsOut_;

    public
    CoreTraceLevels(Logger logger,
                    java.util.Properties properties)
    {
        traceConnections_ = 0;
        traceRetry_ = 0;
	traceRequestsIn_ = 0;
	traceRequestsOut_ = 0;

	String propRoot = "ooc.orb.trace.";
        java.util.Enumeration keys = properties.keys();
	while(keys.hasMoreElements())
	{
	    String key = (String)keys.nextElement();
	    if(!key.startsWith(propRoot))
		continue;

            String value = properties.getProperty(key);
	    Assert._OB_assert(value != null);

	    if(key.equals("ooc.orb.trace.connections"))
	    {
		try
		{
		    traceConnections_ = Integer.parseInt(value);
		}
		catch(NumberFormatException ex)
		{
                    logger.warning("ORB.init: invalid value for " + key);
		}
	    }
	    else if(key.equals("ooc.orb.trace.retry"))
	    {
		try
		{
		    traceRetry_ = Integer.parseInt(value);
		}
		catch(NumberFormatException ex)
		{
                    logger.warning("ORB.init: invalid value for " + key);
		}
	    }
	    else if(key.equals("ooc.orb.trace.requests"))
	    {
		try
		{
		    traceRequestsIn_  = Integer.parseInt(value);
		    traceRequestsOut_ = Integer.parseInt(value);
		}
		catch(NumberFormatException ex)
		{
                    logger.warning("ORB.init: invalid value for " + key);
		}
	    }
	    else if(key.equals("ooc.orb.trace.requests_in"))
	    {
		try
		{
		    traceRequestsIn_ = Integer.parseInt(value);
		}
		catch(NumberFormatException ex)
		{
                    logger.warning("ORB.init: invalid value for " + key);
		}
	    }
	    else if(key.equals("ooc.orb.trace.requests_out"))
	    {
		try
		{
		    traceRequestsOut_ = Integer.parseInt(value);
		}
		catch(NumberFormatException ex)
		{
                    logger.warning("ORB.init: invalid value for " + key);
		}
	    }
	    else
	    {
		logger.warning("ORB.init: unknown property: " + key);
	    }
        }
    }

    public int
    traceConnections()
    {
	return traceConnections_;
    }

    public int
    traceRetry()
    {
	return traceRetry_;
    }

    public int
    traceRequestsIn()
    {
	return traceRequestsIn_;
    }

    public int
    traceRequestsOut()
    {
	return traceRequestsOut_;
    }

}
