// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

import com.ooc.PortableServer.*;

abstract class GIOPServerStarter
{
    protected ORBInstance orbInstance_;         // The ORBInstance
    protected com.ooc.OCI.Acceptor acceptor_;   // The acceptor
    protected OAInterface oaInterface_;         // The OA interface
    protected java.util.Vector connections_ = 
        new java.util.Vector();                 // Workers

    public static final int StateActive  = 0;
    public static final int StateHolding = 1;
    public static final int StateClosed  = 2;

    protected int state_;

    // ----------------------------------------------------------------------
    // GIOPServer private and protected member implementation
    // ----------------------------------------------------------------------

    protected void
    finalize()
	throws Throwable
    {
        Assert._OB_assert(state_ == StateClosed);

        super.finalize();
    }

    //
    // Emit a trace message when closing the acceptor
    //
    protected void
    logCloseAcceptor()
    {
        CoreTraceLevels coreTraceLevels = orbInstance_.getCoreTraceLevels();
        if(coreTraceLevels.traceConnections() > 0)
        {
            com.ooc.OCI.AcceptorInfo info = acceptor_.get_info();
            String msg = "stopped accepting connections\n";
            msg += info.describe();
            orbInstance_.getLogger().trace("incoming", msg);
        }
    }

    protected void
    reapWorkers()
    {
        for(int i = 0 ; i < connections_.size() ;)
        {
            GIOPConnection connection = 
                (GIOPConnection)connections_.elementAt(i);
            if(connection.destroyed())
                connections_.removeElementAt(i);
            else
                ++i;

        }
    }

    // ----------------------------------------------------------------------
    // GIOPServer public member implementation
    // ----------------------------------------------------------------------

    GIOPServerStarter(ORBInstance orbInstance, 
                      com.ooc.OCI.Acceptor acceptor,
		      OAInterface oaInterface)
    {
        orbInstance_ = orbInstance;
	acceptor_ = acceptor;
	oaInterface_ = oaInterface;
        state_ = StateHolding; // Must be holding initially

        try
        {
            //
            // Trace acceptor creation
            //
            CoreTraceLevels coreTraceLevels = 
                orbInstance_.getCoreTraceLevels();
            if(coreTraceLevels.traceConnections() > 0)
            {
                com.ooc.OCI.AcceptorInfo info = acceptor_.get_info();
                String msg = "accepting connections\n";
                msg += info.describe();
                orbInstance_.getLogger().trace("incoming", msg);
            }

            //
            // Start listening
            //
            acceptor_.listen();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            acceptor_.close();
            state_ = StateClosed;
            throw ex;
        }
    }

    //
    // given a host/port this will search the workers of this
    // GIOPServerStarter for a transport which matches the specific
    // connection information.  It returns null if not found.
    //
    public synchronized GIOPConnection 
    getWorker(com.ooc.OCI.ConnectorInfo connInfo)
    {
        //
        // reap the workers first since we don't want to return a
        // destroyed transport
        //
        reapWorkers();
       
        // 
        // iterate the workers 
        //
        for(int i = 0; i < connections_.size(); i++)
        {
            GIOPConnection worker = 
                (GIOPConnection)connections_.elementAt(i);
                
            com.ooc.OCI.Transport transport = worker.transport();

            if(transport != null)
                if(transport.get_info().endpoint_alias_match(connInfo))
                    return worker;
        }
        
        // 
        // we never found a match for the transport
        // 
        return null;
    }
    
    //
    // Change the state of the worker
    //
    abstract public void
    setState(int state);
}

