// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

public final class IORUtil
{
    private static String
    describeCodeSets(org.omg.IOP.TaggedComponent component)
    {
        String result = "";
        byte[] coct = component.component_data;
        int len = component.component_data.length;
        com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer(coct, len);
        com.ooc.CORBA.InputStream in =
            new com.ooc.CORBA.InputStream(buf, 0, false);
        in._OB_readEndian();
        org.omg.CONV_FRAME.CodeSetComponentInfo info =
            org.omg.CONV_FRAME.CodeSetComponentInfoHelper.read(in);

        CodeSetDatabase db = CodeSetDatabase.instance();
        CodeSetInfo charInfo;

        //
        // Print char codeset information
        //
        result += "Native char codeset: \n";
        charInfo = db.getCodeSetInfo(info.ForCharData.native_code_set);
        if(charInfo != null)
        {
            result += "  \"";
            result += charInfo.description;
            result += "\"\n";
        }
        else if(info.ForCharData.native_code_set == 0)
            result += "  [No codeset information]\n";
        else
        {
            result += "  [Unknown codeset id: ";
            result += info.ForCharData.native_code_set;
            result += "]\n";
        }

        for(int i = 0 ; i < info.ForCharData.conversion_code_sets.length ; i++)
        {
            if(i == 0)
                result += "Char conversion codesets:\n";

            charInfo =
                db.getCodeSetInfo(info.ForCharData.conversion_code_sets[i]);
            if(charInfo != null)
            {
                result += "  \"";
                result += charInfo.description;
                result += "\"\n";
            }
            else
            {
                result += "  [Unknown codeset id: ";
                result += info.ForCharData.conversion_code_sets[i];
                result += "]\n";
            }
        }

        //
        // Print wchar codeset information
        //
        result += "Native wchar codeset: \n";
        charInfo = db.getCodeSetInfo(info.ForWcharData.native_code_set);
        if(charInfo != null)
        {
            result += "  \"";
            result += charInfo.description;
            result += "\"\n";
        }
        else if(info.ForWcharData.native_code_set == 0)
            result += "  [No codeset information]\n";
        else
        {
            result += "  [Unknown codeset id: ";
            result += info.ForWcharData.native_code_set;
            result += "]\n";
        }

        for(int i = 0 ; i < info.ForWcharData.conversion_code_sets.length ;
            i++)
        {
            if(i == 0)
                result += "Wchar conversion codesets:\n";

            charInfo =
                db.getCodeSetInfo(info.ForWcharData.conversion_code_sets[i]);
            if(charInfo != null)
            {
                result += "  \"";
                result += charInfo.description;
                result += "\"\n";
            }
            else
            {
                result += "  [Unknown codeset id: ";
                result += info.ForWcharData.conversion_code_sets[i];
                result += "]\n";
            }
        }

        return result;
    }

    private static String
    describeGenericComponent(org.omg.IOP.TaggedComponent component,
                             String name)
    {
        String result = "Component: ";
        result += name;
        result += '\n';
        result += "Component data: (";
        result += component.component_data.length;
        result += ")\n";
        String data = dump_octets(component.component_data, 0,
                                  component.component_data.length);
        result += data;
        return result;
    }

    //
    // Convert an octet buffer into human-friendly data dump
    //
    public static String
    dump_octets(byte[] oct, int offset, int count)
    {
        final int inc = 8;

        StringBuffer result = new StringBuffer((count - offset) * 8);

        for(int i = offset ; i < offset + count ; i += inc)
        {
            for(int j = i ; j - i < inc ; j++)
            {
                if(j < offset + count)
                {
                    int n = (int)oct[j];
                    if(n < 0)
                        n += 256;
                    if(n < 10)
                        result.append("  ");
                    else if(n < 100)
                        result.append(' ');
                    result.append(n);
                    result.append(' ');
                }
                else
                    result.append("    ");
            }

            result.append('"');

            for(int j = i ; j < offset + count && j - i < inc ; j++)
            {
                if(oct[j] >= (byte)32 && oct[j] < (byte)127)
                    result.append((char)oct[j]);
                else
                    result.append('.');
            }

            result.append('"');
            result.append('\n');
        }

        return result.toString();
    }

    //
    // Produce a human-friendly description of an IOR tagged component
    //
    public static String
    describe_component(org.omg.IOP.TaggedComponent component)
    {
        String result;

        switch(component.tag)
        {
        case org.omg.IOP.TAG_ORB_TYPE.value:
        {
            byte[] coct = component.component_data;
            int len = component.component_data.length;
            com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer(coct, len);
            com.ooc.CORBA.InputStream in =
                new com.ooc.CORBA.InputStream(buf, 0, false);
            in._OB_readEndian();
            int id = in.read_ulong();
            result = "Component: TAG_ORB_TYPE = ";
	    result += "0x";
            result += Integer.toHexString(id);
            result += '\n';
            break;
        }

        case org.omg.IOP.TAG_CODE_SETS.value:
            result = describeCodeSets(component);
            break;

        case org.omg.IOP.TAG_POLICIES.value:
            result = describeGenericComponent(component, "TAG_POLICIES");
            break;

        case org.omg.IOP.TAG_ALTERNATE_IIOP_ADDRESS.value:
        {
            byte[] coct = component.component_data;
            int len = component.component_data.length;
            com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer(coct, len);
            com.ooc.CORBA.InputStream in =
                new com.ooc.CORBA.InputStream(buf, 0, false);
            in._OB_readEndian();
            String host = in.read_string();
            short port = in.read_ushort();
            result = "Alternate IIOP address:\n";
            result += "  host: ";
            result += host;
            result += '\n';
            result += "  port: ";
            result += (port < 0 ? 0xffff + (int)port + 1 : port);
            result += '\n';
            break;
        }

        case org.omg.IOP.TAG_ASSOCIATION_OPTIONS.value:
            result = describeGenericComponent(component,
                                              "TAG_ASSOCIATION_OPTIONS");
            break;

        case org.omg.IOP.TAG_SEC_NAME.value:
            result = describeGenericComponent(component, "TAG_SEC_NAME");
            break;

        case org.omg.IOP.TAG_SPKM_1_SEC_MECH.value:
            result = describeGenericComponent(component,
                                              "TAG_SPKM_1_SEC_MECH");
            break;

        case org.omg.IOP.TAG_SPKM_2_SEC_MECH.value:
            result = describeGenericComponent(component,
                                              "TAG_SPKM_2_SEC_MECH");
            break;

        case org.omg.IOP.TAG_KerberosV5_SEC_MECH.value:
            result = describeGenericComponent(component,
                                              "TAG_KerberosV5_SEC_MECH");
            break;

        case org.omg.IOP.TAG_CSI_ECMA_Secret_SEC_MECH.value:
            result = describeGenericComponent(component,
                                              "TAG_CSI_ECMA_Secret_SEC_MECH");
            break;

        case org.omg.IOP.TAG_CSI_ECMA_Hybrid_SEC_MECH.value:
            result = describeGenericComponent(component,
                                              "TAG_CSI_ECMA_Hybrid_SEC_MECH");
            break;

        case org.omg.IOP.TAG_SSL_SEC_TRANS.value:
            result = describeGenericComponent(component, "TAG_SSL_SEC_TRANS");
            break;

        case org.omg.IOP.TAG_CSI_ECMA_Public_SEC_MECH.value:
            result = describeGenericComponent(component,
                                              "TAG_CSI_ECMA_Public_SEC_MECH");
            break;

        case org.omg.IOP.TAG_GENERIC_SEC_MECH.value:
            result = describeGenericComponent(component,
                                              "TAG_GENERIC_SEC_MECH");
            break;

        case org.omg.IOP.TAG_JAVA_CODEBASE.value:
        {
            byte[] coct = component.component_data;
            int len = component.component_data.length;
            com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer(coct, len);
            com.ooc.CORBA.InputStream in =
                new com.ooc.CORBA.InputStream(buf, 0, false);
            in._OB_readEndian();
            String codebase = in.read_string();
            result = "Component: TAG_JAVA_CODEBASE = `";
            result += codebase;
            result += "'\n";
            break;
        }

        case org.omg.IOP.TAG_COMPLETE_OBJECT_KEY.value:
            result = describeGenericComponent(component,
                                              "TAG_COMPLETE_OBJECT_KEY");
            break;

        case org.omg.IOP.TAG_ENDPOINT_ID_POSITION.value:
            result = describeGenericComponent(component,
                                              "TAG_ENDPOINT_ID_POSITION");
            break;

        case org.omg.IOP.TAG_LOCATION_POLICY.value:
            result = describeGenericComponent(component,
                                              "TAG_LOCATION_POLICY");
            break;

        case org.omg.IOP.TAG_DCE_STRING_BINDING.value:
            result = describeGenericComponent(component,
                                              "TAG_DCE_STRING_BINDING");
            break;

        case org.omg.IOP.TAG_DCE_BINDING_NAME.value:
            result = describeGenericComponent(component,
                                              "TAG_DCE_BINDING_NAME");
            break;

        case org.omg.IOP.TAG_DCE_NO_PIPES.value:
            result = describeGenericComponent(component, "TAG_DCE_NO_PIPES");
            break;

        case org.omg.IOP.TAG_DCE_SEC_MECH.value:
            result = describeGenericComponent(component, "TAG_DCE_SEC_MECH");
            break;

        default:
        {
            String name = "unknown (tag = ";
            name += component.tag;
            name += ")";
            result = describeGenericComponent(component, name);
            break;
        }
        }

        return result;
    }
}
