// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

public final class Net
{
    //
    // Compare two hosts strings for equality.  These don't necessarily
    // mean they are string comparable... for instance www.cnn.com and
    // 64.236.24.12 could be the same host
    //
    public static boolean
    CompareHosts(String host1, String host2)
    {
        try
        {
            java.net.InetAddress addr1 = 
                java.net.InetAddress.getByName(host1);
            java.net.InetAddress addr2 = 
                java.net.InetAddress.getByName(host2);
        
            if(addr1.equals(addr2))
                return true;
        }
        catch(java.net.UnknownHostException ex)
        {
            return false;
        }

        return false;
    }
    
    //
    // Get the "canonical" hostname of the local host. If numeric is
    // true, the canonical IP address will be returned instead.
    //
    public static String
    getCanonicalHostname(boolean numeric)
    {
        String host;

        try
        {
            if(!numeric)
            {
                host = 
		    java.net.InetAddress.getLocalHost().getCanonicalHostName();

                if(host.lastIndexOf('.') == -1)
                {
                    //logger.warning("ORB_init: " +
                    //"hostname is a domainless name\n" +
                    //"using dotted decimal notation " +
                    //"instead of hostname");

                    host =
                        java.net.InetAddress.getLocalHost().getHostAddress();
                }
            }
            else
            {
                host = java.net.InetAddress.getLocalHost().getHostAddress();
            }

            if(host.equals("127.0.0.1") || host.equals("localhost"))
            {
                //logger.warning("ORB_init: " +
                //"hostname lookup returned " +
                //"`localhost' (127.0.0.1)\n" +
                //"Use the -IIOPhost option to select " +
                //"some other hostname");
            }
        }
        catch(java.net.UnknownHostException ex)
        {
            //logger.warning("ORB_init: " +
            //"can't resolve hostname\n" +
            //"using `localhost' (127.0.0.1) " +
            //"instead of hostname");

            if(numeric)
                host = "127.0.0.1";
            else
                host = "localhost";
        }

        return host;
    }
}
