// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

public final class ObjectFactory
{
    private boolean destroy_; // True if destroy() was called

    ORBInstance orbInstance_; // The ORBInstance object
    org.omg.CORBA.PolicyManager policyManager_; // The PolicyManager object

    // ----------------------------------------------------------------------
    // ObjectFactory private and protected member implementations
    // ----------------------------------------------------------------------

    protected void
    finalize()
        throws Throwable
    {
        Assert._OB_assert(destroy_);

        super.finalize();
    }

    // ----------------------------------------------------------------------
    // ObjectFactory package member implementations
    // ----------------------------------------------------------------------

    void
    destroy()
    {
        Assert._OB_assert(!destroy_);
        destroy_ = true;

        //
        // Set the ORBInstance object to nil
        //
        orbInstance_ = null;

        //
        // Set the PolicyManager object to nil
        //
        policyManager_ = null;
    }

    // ----------------------------------------------------------------------
    // ObjectFactory public member implementations
    // ----------------------------------------------------------------------

    public void
    setORBInstance(ORBInstance orbInstance)
    {
        orbInstance_ = orbInstance;
    }

    public void
    setPolicyManager(org.omg.CORBA.PolicyManager policyManager)
    {
        policyManager_ = policyManager;
    }

    public org.omg.CORBA.Object
    createObject(org.omg.IOP.IOR ior)
    {
        //
        // The ORB destroys this object, so it's an initialization error
        // if this operation is called after ORB destruction
        //
        if(destroy_)
            throw new org.omg.CORBA.INITIALIZE(
                MinorCodes.describeInitialize(MinorCodes.MinorORBDestroyed),
                MinorCodes.MinorORBDestroyed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        //
        // Check for nil object reference
        //
        if(ior.type_id.length() == 0 && ior.profiles.length == 0)
            return null;

        //
        // Create new delegate, set policies and change delegate
        //
        RefCountPolicyList policyList = new RefCountPolicyList(policies());
        com.ooc.CORBA.Delegate p =
            new com.ooc.CORBA.Delegate(orbInstance_, ior, ior, policyList);

        //
        // Create new object, set the delegate and return
        //
        com.ooc.CORBA.StubForObject obj = new com.ooc.CORBA.StubForObject();
        obj._set_delegate(p);
        return obj;
    }

    public org.omg.CORBA.Object
    stringToObject(String ior)
    {
        return orbInstance_.getURLRegistry().parse_url(ior);
    }

    public org.omg.CORBA.Policy[]
    policies()
    {
	int[] ts = new int[0];
        return policyManager_.get_policy_overrides(ts);
    }
}
