// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

public final class PluginManager
{
    //
    // The ORB
    //
    private org.omg.CORBA.ORB orb_;

    //
    // Plug-in data
    //
    static class PluginData
    {
        public String name;
        public com.ooc.OCI.Plugin plugin;

        PluginData(String name, com.ooc.OCI.Plugin plugin)
        {
            this.name = name;
            this.plugin = plugin;
        }
    }

    //
    // The set of registered plug-ins
    //
    private java.util.Vector plugins_ = new java.util.Vector();

    private boolean destroy_; // True if destroy() was called

    // ----------------------------------------------------------------------
    // PluginManager public member implementations
    // ----------------------------------------------------------------------

    public
    PluginManager(org.omg.CORBA.ORB orb)
    {
        orb_ = orb;
        destroy_ = false;
    }

    //
    // Destroy the PluginManager
    //
    public void
    destroy()
    {
        Assert._OB_assert(!destroy_); // May only be destroyed once
        destroy_ = true;

        //
        // Destroy the plug-ins
        //
        plugins_.removeAllElements();

        //
        // Eliminate circular reference
        //
        orb_ = null;
    }

    //
    // Initialize the plug-in with the given name. The plug-in may be
    // loaded dynamically. If the plug-in could not be initialized,
    // nil is returned.
    //
    public com.ooc.OCI.Plugin
    initPlugin(String name, org.omg.CORBA.StringSeqHolder args)
    {
        com.ooc.OCI.Plugin result = null;

        for(int i = 0 ; i < plugins_.size() ; i++)
        {
            PluginData data = (PluginData)plugins_.elementAt(i);
            if(name.equals(data.name))
            {
                result = data.plugin;
                break;
            }
        }

        if(result == null)
        {
            //
            // Try to load the plug-in dynamically
            //

            String className;

            //
            // First check the properties to see if there is a property
            // with the name ooc.oci.plugin.<name>, which specifies the
            // class from which this plug-in should be loaded
            //
            com.ooc.CORBA.ORB oborb = (com.ooc.CORBA.ORB)orb_;
            java.util.Properties props = oborb.properties();
            String propName = "ooc.oci.plugin." + name;
            className = props.getProperty(propName);

            if(className == null)
            {
                //
                // No property was found, so compose the class name using
                // a standard format: com.ooc.OCI.<name>
                //
                className = "com.ooc.OCI." + name;
            }

            Logger logger = oborb.logger();

            //
            // Load the class
            //
            com.ooc.OCI.PluginInit pi = null;
            try
            {
                Class c = Class.forName(className);
                pi = (com.ooc.OCI.PluginInit)c.newInstance();
            }
            catch(org.omg.CORBA.SystemException ex)
            {
                throw ex;
            }
            catch(Exception ex)
            {
                String err = "unable to load OCI plug-in `" + name + "':\n" +
                    ex.getMessage();
                logger.error(err);
                return null;
            }

            //
            // Invoke the version function, which allows the plug-in
            // to verify that it is compatible with the OCI version
            // in use by the ORB
            //
            pi.version(orb_, com.ooc.OCI.Version.value);

            //
            // Invoke the initialization function
            //
            result = pi.init(orb_, args);
            if(result != null)
                plugins_.addElement(new PluginData(name, result));
        }

        return result;
    }
}
