// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

public final class ServerManager
{
    private boolean destroy_;                   // if destroy() was called
    private CollocatedServer collocatedServer_; // The collocated server
    private java.util.Vector allServers_ = 
        new java.util.Vector();                 // all other servers

    // ----------------------------------------------------------------------
    // ServerManager private and protected member implementations
    // ----------------------------------------------------------------------

    protected void
    finalize()
	throws Throwable
    {
        Assert._OB_assert(destroy_);
        Assert._OB_assert(allServers_.isEmpty());
        Assert._OB_assert(collocatedServer_ == null);

        super.finalize();
    }

    // ----------------------------------------------------------------------
    // ServerManager public member implementations
    // ----------------------------------------------------------------------

    public
    ServerManager(ORBInstance orbInstance,
                  com.ooc.OCI.Acceptor[] acceptors,
                  com.ooc.OB.OAInterface oaInterface,
                  int concModel)
    {
        destroy_ = false;

	//
	// Create a server for each acceptor, and the collocated server
	//
	for(int i = 0 ; i < acceptors.length ; i++)
	{
	    GIOPServer server = new GIOPServer(orbInstance, acceptors[i],
                                               oaInterface, concModel);
	    allServers_.addElement(server);
	}
	collocatedServer_ = new CollocatedServer(oaInterface, concModel);
        allServers_.addElement(collocatedServer_);
    }

    public synchronized void
    destroy()
    {
        //
        // Don't destroy twice
        //
        if(destroy_)
            return;

        //
        // Set the destroy flag
        //
	destroy_ = true;

	//
	// Destroy all servers
	//
	java.util.Enumeration e = allServers_.elements();
	while(e.hasMoreElements())
	    ((Server)e.nextElement()).destroy();

        allServers_.removeAllElements();
	collocatedServer_ = null;
    }

    public synchronized void
    hold()
    {
	java.util.Enumeration e = allServers_.elements();
	while(e.hasMoreElements())
	    ((Server)e.nextElement()).hold();
    }

    public synchronized void
    activate()
    {
	java.util.Enumeration e = allServers_.elements();
	while(e.hasMoreElements())
	    ((Server)e.nextElement()).activate();
    }

    public synchronized CollocatedServer
    getCollocatedServer()
    {
	return collocatedServer_;
    }

    public synchronized com.ooc.OB.Server[]
    getServers()
    {
        com.ooc.OB.Server[] servers = 
            new com.ooc.OB.Server[allServers_.size()];

        for(int i = 0; i < allServers_.size(); i++)
            servers[i] = (com.ooc.OB.Server)allServers_.elementAt(i);
        
        return servers;
    }
        
}
