// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

public final class ValueFactoryManager
{
    //
    // The set of registered valuetype factories
    //
    private java.util.Hashtable factories_;

    //
    // Cached set of factories resolved by class (Java only)
    //
    private java.util.Hashtable classFactories_;

    private boolean destroy_; // True if destroy() was called

    // ----------------------------------------------------------------------
    // ValueFactoryManager private and protected member implementations
    // ----------------------------------------------------------------------

    protected void
    finalize()
        throws Throwable
    {
        Assert._OB_assert(destroy_);

	super.finalize();
    }

    // ----------------------------------------------------------------------
    // ValueFactoryManager package member implementations
    // ----------------------------------------------------------------------

    synchronized void
    destroy()
    {
        Assert._OB_assert(!destroy_); // May only be destroyed once

        //
        // Destroy the hashtable
        //
        factories_ = null;
    }

    // ----------------------------------------------------------------------
    // ValueFactoryManager public member implementations
    // ----------------------------------------------------------------------

    public
    ValueFactoryManager()
    {
        //
        // Create the hashtables
        //
        factories_ = new java.util.Hashtable(1023);
        classFactories_ = new java.util.Hashtable(1023);

        //
        // Install factories for standard value box types
        //
        registerValueFactory(org.omg.CORBA.StringValueHelper.id(),
                             new com.ooc.CORBA.StringValueFactory());
        registerValueFactory(org.omg.CORBA.WStringValueHelper.id(),
                             new com.ooc.CORBA.WStringValueFactory());
    }

    public synchronized org.omg.CORBA.portable.ValueFactory
    registerValueFactory(String id,
                         org.omg.CORBA.portable.ValueFactory factory)
    {
        //
        // The ORB destroys this object, so it's an initialization error
        // if this operation is called after ORB destruction
        //
        if(destroy_)
            throw new org.omg.CORBA.INITIALIZE(
                MinorCodes.describeInitialize(MinorCodes.MinorORBDestroyed),
                MinorCodes.MinorORBDestroyed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        Assert._OB_assert(id != null && factory != null);

        org.omg.CORBA.portable.ValueFactory old =
            (org.omg.CORBA.portable.ValueFactory)factories_.get(id);

        factories_.put(id, factory);

        return old;
    }

    public synchronized void
    unregisterValueFactory(String id)
    {
        //
        // The ORB destroys this object, so it's an initialization error
        // if this operation is called after ORB destruction
        //
        if(destroy_)
            throw new org.omg.CORBA.INITIALIZE(
                MinorCodes.describeInitialize(MinorCodes.MinorORBDestroyed),
                MinorCodes.MinorORBDestroyed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        Assert._OB_assert(id != null);

        if(factories_.remove(id) == null)
            throw new org.omg.CORBA.BAD_PARAM(
                MinorCodes.describeBadParam(MinorCodes.MinorValueFactoryError)
                    + ": " + id,
                MinorCodes.MinorValueFactoryError,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
    }

    public synchronized org.omg.CORBA.portable.ValueFactory
    lookupValueFactory(String id)
    {
        //
        // The ORB destroys this object, so it's an initialization error
        // if this operation is called after ORB destruction
        //
        if(destroy_)
            throw new org.omg.CORBA.INITIALIZE(
                MinorCodes.describeInitialize(MinorCodes.MinorORBDestroyed),
                MinorCodes.MinorORBDestroyed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        Assert._OB_assert(id != null);

        return (org.omg.CORBA.portable.ValueFactory)factories_.get(id);
    }

    // Java-specific method
    public org.omg.CORBA.portable.ValueFactory
    lookupValueFactoryWithClass(String id)
    {
        //
        // The ORB destroys this object, so it's an initialization error
        // if this operation is called after ORB destruction
        //
        if(destroy_)
            throw new org.omg.CORBA.INITIALIZE(
                MinorCodes.describeInitialize(MinorCodes.MinorORBDestroyed),
                MinorCodes.MinorORBDestroyed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        Assert._OB_assert(id != null);

        org.omg.CORBA.portable.ValueFactory result;

        //
        // Check the registered factories
        //
        result = (org.omg.CORBA.portable.ValueFactory)factories_.get(id);
        if(result != null)
            return result;

        //
        // Check the cached factories
        //
        result = (org.omg.CORBA.portable.ValueFactory)classFactories_.get(id);
        if(result != null)
            return result;

        //
        // Try to convert the repository ID into a class name.
        //
        Class c = Util.idToClass(id, "DefaultFactory");
        if(c != null)
        {
            try
            {
                //
                // Instantiate the factory
                //
                result = (org.omg.CORBA.portable.ValueFactory)c.newInstance();

                //
                // Cache the result
                //
                classFactories_.put(id, result);
            }
            catch(ClassCastException ex)
            {
                // ignore
            }
            catch(InstantiationException ex)
            {
                // ignore
            }
            catch(IllegalAccessException ex)
            {
                // ignore
            }
        }

        return result;
    }
}
