// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OBPortableInterceptor;

//
// The Transient ObjectReferenceTemplate
//
final public class TransientORT_impl extends TransientORT
{
    //
    // The ORBInstance object
    //
    private com.ooc.OB.ORBInstance orbInstance_;
    
    // ------------------------------------------------------------------
    // Public member implementations
    // ------------------------------------------------------------------

    public 
    TransientORT_impl(com.ooc.OB.ORBInstance orbInstance)
    {
        orbInstance_ = orbInstance;
    }

    public
    TransientORT_impl(
        com.ooc.OB.ORBInstance orbInstance,
	String serverId,
	String orbId,
        String[] adapterName,
	int createTime,
   	org.omg.IOP.IOR iorTemplate)
    {
        orbInstance_ = orbInstance;

	the_server_id = serverId;
	the_orb_id = orbId;
	the_adapter_name = adapterName;
	the_create_time = createTime;
	the_ior_template = iorTemplate;
    }

    // ------------------------------------------------------------------
    // Private member implementation
    // ------------------------------------------------------------------

    private org.omg.CORBA.Object
    makeObject(String repoid, byte[] id, String[] name)
    {
        //
        // Create the actual ORBacus ObjectKey
        //
        // TODO:
        // CreatePersistentObjectKey/CreateTransientObjectKey instead of
        // populating this ObjectKey data to avoid the copy?
        //
        com.ooc.OB.ObjectKeyData obkey = new com.ooc.OB.ObjectKeyData();
        obkey.serverId = the_server_id;
	if(obkey.serverId.length() == 0)
	    obkey.serverId = "_RootPOA";
        obkey.poaId = name;
        obkey.oid = id;
        obkey.persistent = false;
        obkey.createTime = the_create_time;
    
        byte[] key = com.ooc.OB.ObjectKey.CreateObjectKey(obkey);

        org.omg.IOP.IOR ior = new org.omg.IOP.IOR();
        ior.type_id = repoid;
        ior.profiles =
            new org.omg.IOP.TaggedProfile[the_ior_template.profiles.length];
        for(int profile = 0 ;
            profile < the_ior_template.profiles.length ;
            ++profile)
        {
            ior.profiles[profile] = new org.omg.IOP.TaggedProfile();
            ior.profiles[profile].tag = the_ior_template.profiles[profile].tag;
            ior.profiles[profile].profile_data = new byte[
                the_ior_template.profiles[profile].profile_data.length];
            System.arraycopy(the_ior_template.profiles[profile].profile_data,
                             0, ior.profiles[profile].profile_data, 0,
                             ior.profiles[profile].profile_data.length);
        }
	org.omg.IOP.IORHolder iorH = new org.omg.IOP.IORHolder(ior);

        com.ooc.OCI.AccFactoryRegistry registry =
            orbInstance_.getAccFactoryRegistry();
        com.ooc.OCI.AccFactory[] factories = registry.get_factories();
        for(int i = 0 ; i < factories.length ; ++i)
            factories[i].change_key(iorH, key);

        //
        // Create and return reference
        //
        com.ooc.OB.ObjectFactory objectFactory =
	    orbInstance_.getObjectFactory();
        return objectFactory.createObject(iorH.value);
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java mapping
    // ------------------------------------------------------------------

    public String
    server_id()
    {
        return the_server_id;
    }

    public String
    orb_id()
    {
        return the_orb_id;
    }

    public String[]
    adapter_name()
    {
	String[] result = new String[the_adapter_name.length];
	System.arraycopy(the_adapter_name, 0, result, 0, 
			 the_adapter_name.length);
	return result;
    }

    public org.omg.CORBA.Object
    make_object(String repoid, byte[] id)
    {
        return makeObject(repoid, id, the_adapter_name);
    }

    public org.omg.CORBA.Object
    make_object_for(String repoid, byte[] id, String[] adapterName)
    {
        return makeObject(repoid, id, adapterName);
    }
}
