// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OBPortableServer;

public class DirectServant extends org.omg.CORBA.portable.ServantObject
{
    //
    // The POA
    //
    private com.ooc.OBPortableServer.POA_impl poa_;

    // The object ID
    //
    private byte[] oid_;

    //
    // This flag is true if the servant has been deactivated
    //
    private boolean deactivated_;

    public
    DirectServant(com.ooc.OBPortableServer.POA_impl poa,
                  byte[] oid, java.lang.Object servant)
    {
        poa_ = poa;
        oid_ = oid;
        deactivated_ = false;
        this.servant = servant;
    }

    protected void
    finalize()
        throws Throwable
    {
        //
        // This object *must* have been deactivated already
        //
        com.ooc.OB.Assert._OB_assert(deactivated_);

        super.finalize();
    }

    public void
    destroy()
    {
        //
        // An explicit destroy method is needed in Java to force the
        // removal of this object from the POA's table. Otherwise,
        // this object will never be garbage collected.
        //
        if(!deactivated_)
        {
            poa_._OB_removeDirectServant(oid_, this);
            deactivated_ = true;
        }
    }

    public boolean
    deactivated()
    {
	return deactivated_;
    }

    public void
    deactivate()
    {
	deactivated_ = true;
    }

    public boolean
    preinvoke(String op)
    {
        //
        // Validate POA manager state
        //
        poa_._OB_validateManagerState();

        //
        // Increment the POA's request count
        //
        if(!poa_._OB_incrementRequestCount())
            return false;

        //
        // Preinvoke
        //
        poa_._OB_preinvoke(op, oid_, (org.omg.PortableServer.Servant)servant,
                           null, null);

        return true;
    }

    public void
    postinvoke()
    {
        //
        // Postinvoke
        //
        poa_._OB_postinvoke();

        //
        // Decrement the outstanding request count
        //
        poa_._OB_decrementRequestCount();
    }

    public boolean
    locate_request()
    {
        try
        {
            if(preinvoke("_locate"))
            {
                postinvoke();
                return true;
            }
        }
        catch(org.omg.CORBA.OBJECT_NOT_EXIST ex)
        {
            // Fall through
        }

        return false;
    }
}
