// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OBPortableServer;

//
// Strategy for NON_RETAIN and USE_SERVANT_MANAGER or
// USE_DEFAULT_SERVANT
//
class NonRetainStrategy implements ServantLocationStrategy
{
    private ServantLocatorStrategy servantManager_;

    private DefaultServantHolder defaultServant_;

    NonRetainStrategy(ServantLocatorStrategy servantManager,
		      DefaultServantHolder defaultServant)
    {
	servantManager_ = servantManager;
	defaultServant_ = defaultServant;
    }
    
    public void
    destroy(org.omg.PortableServer.POA poa, boolean etherealize)
    {
	if(servantManager_ != null)
	    servantManager_.destroy();
	if(defaultServant_ != null)
	    defaultServant_.destroy();
    }

    public void
    etherealize(org.omg.PortableServer.POA poa)
    {
	// Do nothing
    }
    
    public void
    activate(byte[] oid, org.omg.PortableServer.Servant servant)
	throws org.omg.PortableServer.POAPackage.ServantAlreadyActive,
               org.omg.PortableServer.POAPackage.WrongPolicy,
	       org.omg.PortableServer.POAPackage.ObjectAlreadyActive
    {
	//
	// Requires the RETAIN policy.
	//
	throw new org.omg.PortableServer.POAPackage.WrongPolicy();
    }

    public void
    deactivate(org.omg.PortableServer.POA poa, byte[] oid)
	throws org.omg.PortableServer.POAPackage.ObjectNotActive,
               org.omg.PortableServer.POAPackage.WrongPolicy
    {
	//
	// Requires the RETAIN policy.
	//
	throw new org.omg.PortableServer.POAPackage.WrongPolicy();
    }
    
    public byte[]
    servantToId(org.omg.PortableServer.Servant servant,
		com.ooc.PortableServer.Current_impl poaCurrent)
    {
	if(defaultServant_ != null)
	    return defaultServant_.servantToId(servant, poaCurrent);
	return null;
    }
    
    public org.omg.PortableServer.Servant
    idToServant(byte[] oid, boolean useDefaultServant)
    {
	if(defaultServant_ != null)
	    return defaultServant_.getDefaultServant();
	return null;
    }
    
    public org.omg.PortableServer.Servant
    locate(byte[] oid, org.omg.PortableServer.POA poa, String op,
	   org.omg.PortableServer.ServantLocatorPackage.CookieHolder cookie)
	throws com.ooc.OB.LocationForward
    {
	if(defaultServant_ != null)
	{
	    org.omg.PortableServer.Servant servant =
                defaultServant_.getDefaultServant();
	    if(servant == null)
                throw new org.omg.CORBA.OBJ_ADAPTER(
                    com.ooc.OB.MinorCodes.describeObjAdapter(
                        com.ooc.OB.MinorCodes.MinorNoDefaultServant),
                    com.ooc.OB.MinorCodes.MinorNoDefaultServant,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	    return servant;
	}
	
	if(servantManager_ == null)
            throw new org.omg.CORBA.OBJ_ADAPTER(
                com.ooc.OB.MinorCodes.describeObjAdapter(
                    com.ooc.OB.MinorCodes.MinorNoServantManager),
                com.ooc.OB.MinorCodes.MinorNoServantManager,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	
	return servantManager_.preinvoke(oid, poa, op, cookie);
    }

    public void
    preinvoke(byte[] oid)
    {
    }

    public void
    postinvoke(byte[] oid, org.omg.PortableServer.POA poa, String op,
	       java.lang.Object cookie,
	       org.omg.PortableServer.Servant servant)
    {
	if(servantManager_ != null)
	    servantManager_.postinvoke(oid, poa, op, cookie, servant);
    }

    public DirectServant
    createDirectStubImpl(org.omg.PortableServer.POA poa, byte[] oid,
    			 com.ooc.OB.RefCountPolicyList policies)
	throws com.ooc.OB.LocationForward
    {
	return null;
    }

    public void
    removeDirectStubImpl(byte[] oid, DirectServant servant)
    {
    }

    public ServantManagerStrategy
    getServantManagerStrategy()
    {
	return servantManager_;
    }

    public DefaultServantHolder
    getDefaultServantHolder()
    {
	return defaultServant_;
    }
}
