// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OBPortableServer;

//
// Strategy for ServantActivators
//
class ServantActivatorStrategy implements ServantManagerStrategy
{
    //
    // The servant activator
    //
    org.omg.PortableServer.ServantActivator servantActivator_;

    synchronized public void
    destroy()
    {
	servantActivator_ = null;
    }

    synchronized public void
    setServantManager(org.omg.PortableServer.ServantManager manager)
    {
	//
	// Attempting to set the servant manager after one has already
	// been set will result in the BAD_INV_ORDER exception being
	// raised.
	//
	if(servantActivator_ != null)
            throw new org.omg.CORBA.BAD_INV_ORDER(
                com.ooc.OB.MinorCodes.describeBadInvOrder(
                    com.ooc.OB.MinorCodes.MinorServantManagerAlreadySet),
                com.ooc.OB.MinorCodes.MinorServantManagerAlreadySet,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        try
        {
            servantActivator_ =
                org.omg.PortableServer.ServantActivatorHelper.narrow(manager);
        }
        catch(org.omg.CORBA.BAD_PARAM ex)
        {
	}

	if(servantActivator_ == null)
	{
	    throw new org.omg.CORBA.OBJ_ADAPTER(
                com.ooc.OB.MinorCodes.describeObjAdapter(
                    com.ooc.OB.MinorCodes.MinorNoServantManager),
                com.ooc.OB.MinorCodes.MinorNoServantManager,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
    }

    synchronized public org.omg.PortableServer.ServantManager
    getServantManager()
    {
	return servantActivator_;
    }

    //
    // Note that the synchronization in these methods ensures that
    // concurrent calls to the incarnate and etherealize do not occur
    //
    synchronized public org.omg.PortableServer.Servant
    incarnate(byte[] oid, org.omg.PortableServer.POA poa)
	throws com.ooc.OB.LocationForward
    {
	//
	// If no servant manager has been associated with the POA,
	// OBJ_ADAPTER is raised
	//
	if(servantActivator_ == null)
            throw new org.omg.CORBA.BAD_INV_ORDER(
                com.ooc.OB.MinorCodes.describeBadInvOrder(
                    com.ooc.OB.MinorCodes.MinorServantManagerAlreadySet),
                com.ooc.OB.MinorCodes.MinorServantManagerAlreadySet,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

	org.omg.PortableServer.Servant servant;

	try
	{
	    servant = servantActivator_.incarnate(oid, poa);
	}
	catch(org.omg.PortableServer.ForwardRequest ex)
	{
	    com.ooc.CORBA.Delegate p = (com.ooc.CORBA.Delegate)
		(((org.omg.CORBA.portable.ObjectImpl)
		  ex.forward_reference)._get_delegate());
	    org.omg.IOP.IOR ior = p._OB_IOR();
	    throw new com.ooc.OB.LocationForward(ior, false);
	}
	
	//
	// 11-23:
	//
	// If a ServantManager returns a null Servant (or the
	// equivalent in a language mapping) as the result of an
	// incarnate() or preinvoke() operation, the POA will return
	// the OBJ_ADAPTER system exception as the result of the
	// request.
	//
	if(servant == null)
            throw new org.omg.CORBA.OBJ_ADAPTER(
                com.ooc.OB.MinorCodes.describeObjAdapter(
                    com.ooc.OB.MinorCodes.MinorServantNotFound),
                com.ooc.OB.MinorCodes.MinorServantNotFound,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

	return servant;
    }

    synchronized public void
    etherealize(
	byte[] oid,
	org.omg.PortableServer.POA poa,
	org.omg.PortableServer.Servant servant,
	boolean cleanup,
	boolean remaining)
    {
	if(servantActivator_ != null)
	{
	    try
	    {
		servantActivator_.etherealize(oid, poa, servant, cleanup,
					      remaining);
	    }
	    catch(org.omg.CORBA.SystemException ex)
	    {
                // Ignore
	    }
	}
    }
}
