// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OCI;

public final class Buffer
{
    private int max_; // The maximum size of the buffer
    public byte[] data_; // The octet buffer
    public int len_; // The requested size of the buffer
    public int pos_; // The position counter

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public byte[]
    data()
    {
	return data_;
    }

    public int
    length()
    {
	return len_;
    }

    public int
    rest_length()
    {
	return len_ - pos_;
    }

    public int
    pos()
    {
	return pos_;
    }

    public void
    pos(int pos)
    {
	pos_ = pos;
    }

    public void
    advance(int delta)
    {
	pos_ += delta;
    }

    public boolean
    is_full()
    {
	return pos_ >= len_;
    }

    // ------------------------------------------------------------------
    // Additional ORBacus specific functions
    // ------------------------------------------------------------------

    public void
    alloc(int len)
    {
        max_ = len;
        len_ = len;
        try
        {
            data_ = new byte[max_];
        }
        catch(OutOfMemoryError ex)
        {
            throw new org.omg.CORBA.NO_MEMORY(
                com.ooc.OB.MinorCodes.describeNoMemory(
                    com.ooc.OB.MinorCodes.MinorAllocationFailure),
                com.ooc.OB.MinorCodes.MinorAllocationFailure,
                org.omg.CORBA.CompletionStatus.COMPLETED_MAYBE);
        }
        pos_ = 0;
    }

    public void
    realloc(int len)
    {
        if(data_ == null)
            alloc(len);
        else
        {
            com.ooc.OB.Assert._OB_assert(len >= len_);
            if(len <= max_)
                len_ = len;
            else
            {
                int newMax = len > 2 * max_ ? len : 2 * max_;
                byte[] newData = null;
                try
                {
                    newData = new byte[newMax];
                }
                catch(OutOfMemoryError ex)
                {
                    throw new org.omg.CORBA.NO_MEMORY(
                        com.ooc.OB.MinorCodes.describeNoMemory(
                            com.ooc.OB.MinorCodes.MinorAllocationFailure),
                        com.ooc.OB.MinorCodes.MinorAllocationFailure,
                        org.omg.CORBA.CompletionStatus.COMPLETED_MAYBE);
                }
                System.arraycopy(data_, 0, newData, 0, len_);
                data_ = newData;
                len_ = len;
                max_ = newMax;
            }
        }
    }

    public void
    data(byte[] data, int len)
    {
        data_ = data;
        len_ = len;
        max_ = len;
        pos_ = 0;
    }

    public void
    consume(Buffer buf)
    {
        data_ = buf.data_;
        len_ = buf.len_;
        max_ = buf.max_;
        pos_ = buf.pos_;
        buf.data_ = null;
        buf.len_ = 0;
        buf.max_ = 0;
        buf.pos_ = 0;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    public
    Buffer()
    {
    }

    public
    Buffer(byte[] data, int len)
    {
        data_ = data;
        len_ = len;
        max_ = len;
        pos_ = 0;
    }

    public
    Buffer(int len)
    {
        alloc(len);
    }
}
