// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OCI;

public class iiop implements PluginInit
{
    //
    // Compatibility check. The plug-in should verify that it is
    // compatible with the given OCI version, and raise an exception
    // if not.
    //
    public void
    version(org.omg.CORBA.ORB orb, String ver)
    {
        // Nothing to do
    }

    //
    // Initialize the plug-in for an ORB
    //
    public com.ooc.OCI.Plugin
    init(org.omg.CORBA.ORB orb, org.omg.CORBA.StringSeqHolder args)
    {
        com.ooc.CORBA.ORB oborb = (com.ooc.CORBA.ORB)orb;
        java.util.Properties props = oborb.properties();

        //
        // Parse arguments
        //
        args.value = parse_args(args.value, props);

        return new com.ooc.OCI.IIOP.Plugin_impl(orb);
    }

    //
    // Parse IIOP arguments. The return value is the a new array
    // with the IIOP arguments removed.
    //
    public static String[]
    parse_args(String[] args, java.util.Properties props)
    {
        String backlog = null;
        String bind = null;
        String host = null;
        boolean numeric = false;
        String port = null;
        boolean haveArgs = false;

        //
        // First check deprecated properties, which have lowest precedence
        //
        {
            java.util.Enumeration keys = props.keys();
            while(keys.hasMoreElements())
            {
                String key = (String)keys.nextElement();
                if(!key.startsWith("ooc.iiop."))
                    continue;
                String value = props.getProperty(key);

                if(key.equals("ooc.iiop.host"))
                {
                    host = value;
                    haveArgs = true;
                }
                else if(key.equals("ooc.iiop.numeric"))
                {
                    numeric = true;
                    haveArgs = true;
                }
                else if(key.equals("ooc.iiop.port"))
                {
                    port = value;
                    haveArgs = true;
                }
                else
                    throw new org.omg.CORBA.INITIALIZE("iiop: unknown " +
                                                       "property " + key);
            }
        }

        //
        // Check command-line arguments
        //
        int i = 0;
        while(i < args.length)
        {
            if(args[i].equals("-IIOPbacklog"))
            {
                if(i + 1 >= args.length)
                    throw new org.omg.CORBA.INITIALIZE("iiop: argument " +
                                                       "expected for " +
                                                       "-IIOPbacklog");
                backlog = args[i + 1];
                haveArgs = true;
                i += 2;
            }
            else if(args[i].equals("-IIOPbind"))
            {
                if(i + 1 >= args.length)
                    throw new org.omg.CORBA.INITIALIZE("iiop: argument " +
                                                       "expected for " +
                                                       "-IIOPbind");
                bind = args[i + 1];
                haveArgs = true;
                i += 2;
            }
            else if(args[i].equals("-IIOPhost") ||
                    args[i].equals("-OAhost"))
            {
                if(i + 1 >= args.length)
                    throw new org.omg.CORBA.INITIALIZE("iiop: argument " +
                                                       "expected for " +
                                                       args[i]);
                host = args[i + 1];
                haveArgs = true;
                i += 2;
            }
            else if(args[i].equals("-IIOPnumeric") ||
                    args[i].equals("-OAnumeric"))
            {
                numeric = true;
                haveArgs = true;
                i++;
            }
            else if(args[i].equals("-IIOPport") ||
                    args[i].equals("-OAport"))
            {
                if(i + 1 >= args.length)
                    throw new org.omg.CORBA.INITIALIZE("iiop: argument " +
                                                       "expected for " +
                                                       args[i]);
                port = args[i + 1];
                haveArgs = true;
                i += 2;
            }
            else if(args[i].startsWith("-IIOP"))
            {
                throw new org.omg.CORBA.INITIALIZE("iiop: unknown option `" +
                                                   args[i] + "'");
            }
            else
                i++;
        }

        if(haveArgs)
        {
            String propName = "ooc.orb.poamanager.RootPOAManager.endpoint";
            String value = props.getProperty(propName);
            if(value == null)
            {
                propName = "ooc.orb.oa.endpoint";
                value = props.getProperty(propName);
            }

            String str = "iiop";
            if(backlog != null)
            {
                str += " --backlog ";
                str += backlog;
            }
            if(bind != null)
            {
                str += " --bind ";
                str += bind;
            }
            if(host != null)
            {
                str += " --host ";
                //
                // If host contains a comma, then we must put the value
                // in quotes
                //
                if(host.indexOf(',') != -1)
                {
                    str += '"';
                    str += host;
                    str += '"';
                }
                else
                    str += host;
            }
            if(numeric)
            {
                str += " --numeric";
            }
            if(port != null)
            {
                str += " --port ";
                str += port;
            }

            if(value == null)
                props.put(propName, str);
            else
            {
                //
                // Append to existing property value
                //
                props.put(propName, value + ", " + str);
            }
        }

        //
        // Filter arguments
        //
        com.ooc.OB.OptionFilter filter;
        filter = new com.ooc.OB.OptionFilter("iiop.parse_args", "-OA");
        filter.add("host", 1); // Deprecated
        filter.add("numeric", 0); // Deprecated
        filter.add("port", 1); // Deprecated
        args = filter.filter(args);
        filter = new com.ooc.OB.OptionFilter("iiop.parse_args", "-IIOP");
        filter.add("backlog", 1);
        filter.add("bind", 1);
        filter.add("host", 1);
        filter.add("numeric", 0);
        filter.add("port", 1);
        return filter.filter(args);
    }
}
