// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

//
// IDL:ooc.com/OB/CorbalocProtocol:1.0
//
/**
 *
 * The <code>corbaloc</code> URL scheme supports multiple protocols,
 * the most common of which is <code>iiop</code>. The CorbalocURLScheme
 * delegates the parsing of protocol address information to a
 * registered CorbalocProtocol object.
 *
 * @see CorbalocURLScheme
 *
 **/

public interface CorbalocProtocolOperations
{
    //
    // IDL:ooc.com/OB/CorbalocProtocol/name:1.0
    //
    /**
     *
     * Each protocol must have a unique name. All protocol names must
     * be in lower case.
     *
     **/

    String
    name();

    //
    // IDL:ooc.com/OB/CorbalocProtocol/parse_address:1.0
    //
    /**
     *
     * Parse a protocol address and create a tagged profile for inclusion
     * in an IOR.
     *
     * @param addr The protocol address, not including the protocol name
     * or trailing object key. For example, the URL
     * <code>corbaloc:iiop:1.2@localhost:5000/Key</code> would result in
     * an address argument of <code>1.2@localhost:5000</code>.
     *
     * @param key The object key from the URL, converted into a sequence
     * of octets.
     *
     * @return A tagged profile.
     *
     * @exception BAD_PARAM In case the address is invalid.
     *
     **/

    org.omg.IOP.TaggedProfile
    parse_address(String addr,
                  byte[] key);

    //
    // IDL:ooc.com/OB/CorbalocProtocol/destroy:1.0
    //
    /**
     *
     * Release any resources held by the object.
     *
     **/

    void
    destroy();
}
