// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

//
// IDL:ooc.com/OB/DispatchStrategyFactory:1.0
//
/**
 *
 * This interface is a factory to create dispatch strategies, and to
 * manage thread pools.
 *
 * @see DispatchStrategy
 *
 **/

public interface DispatchStrategyFactoryOperations
{
    //
    // IDL:ooc.com/OB/DispatchStrategyFactory/create_thread_pool:1.0
    //
    /**
     *
     * Create a thread pool containing nthreads.
     *
     * @param nthreads The number of threads the pool in the pool
     *
     **/

    int
    create_thread_pool(int nthreads);

    //
    // IDL:ooc.com/OB/DispatchStrategyFactory/destroy_thread_pool:1.0
    //
    /**
     *
     * Destroy a thread pool with the given id. If the thread pool is
     * is use by an object adapter any new requests will cause an
     * <code>OBJ_ADAPTER</code> exception.
     *
     * @param id The thread pool id
     *
     * @exception com.ooc.OB.InvalidThreadPool If the thread pool id is valid.
     *
     **/

    void
    destroy_thread_pool(int id)
        throws InvalidThreadPool;

    //
    // IDL:ooc.com/OB/DispatchStrategyFactory/create_thread_pool_strategy:1.0
    //
    /**
     *
     * Create a thread pool dispatch strategy.
     *
     * @param id The thread pool id
     *
     * @return A dispatch strategy
     *
     * @exception com.ooc.OB.InvalidThreadPool If the thread pool id is valid.
     *
     **/

    DispatchStrategy
    create_thread_pool_strategy(int id)
        throws InvalidThreadPool;

    //
    // IDL:ooc.com/OB/DispatchStrategyFactory/create_same_thread_strategy:1.0
    //
    /**
     *
     * Create a same thread dispatch strategy.
     *
     * @return A dispatch strategy
     *
     **/

    DispatchStrategy
    create_same_thread_strategy();

    //
    // IDL:ooc.com/OB/DispatchStrategyFactory/create_thread_per_request_strategy:1.0
    //
    /**
     *
     * Create a same thread per request dispatch strategy.
     *
     * @return A dispatch strategy
     *
     **/

    DispatchStrategy
    create_thread_per_request_strategy();

    //
    // IDL:ooc.com/OB/DispatchStrategyFactory/create_default_dispatch_strategy:1.0
    //
    /**
     *
     * Create a default dispatch strategy. The default dispatch
     * strategy is created according to the ooc.orb.oa.conc_model
     * property.
     *
     * @return A dispatch strategy
     *
     **/

    DispatchStrategy
    create_default_dispatch_strategy();
}
