// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OCI;

//
// IDL:ooc.com/OCI/ConFactory:1.0
//
/**
 *
 * A factory for Connector objects.
 *
 * @see Connector
 * @see ConFactoryRegistry
 *
 **/

public interface ConFactoryOperations
{
    //
    // IDL:ooc.com/OCI/ConFactory/id:1.0
    //
    /** The plugin id. */

    String
    id();

    //
    // IDL:ooc.com/OCI/ConFactory/tag:1.0
    //
    /** The profile id tag. */

    int
    tag();

    //
    // IDL:ooc.com/OCI/ConFactory/describe_profile:1.0
    //
    /**
     *
     * Returns a description of the given tagged profile.
     *
     * @param prof The tagged profile.
     *
     * @return The profile description.
     *
     **/

    String
    describe_profile(org.omg.IOP.TaggedProfile prof);

    //
    // IDL:ooc.com/OCI/ConFactory/create_connectors:1.0
    //
    /**
     *
     * Returns a sequence of Connectors for a given IOR and a list of
     * policies. The sequence includes one or more Connectors for each
     * IOR profile that matches this Connector factory and satisfies
     * the list of policies.
     *
     * @param ref The IOR for which Connectors are returned.
     *
     * @param policies The policies that must be satisfied.
     *
     * @return The sequence of Connectors.
     *
     **/

    Connector[]
    create_connectors(org.omg.IOP.IOR ref,
                      org.omg.CORBA.Policy[] policies);

    //
    // IDL:ooc.com/OCI/ConFactory/equivalent:1.0
    //
    /**
     *
     * Checks whether two IORs are equivalent, taking only profiles
     * into account matching this Connector factory.
     *
     * @param ior1 The first IOR to check for equivalence.
     *
     * @param ior2 The second IOR to check for equivalence.
     *
     * @return <code>TRUE</code> if the IORs are equivalent,
     * <code>FALSE</code> otherwise.
     *
     **/

    boolean
    equivalent(org.omg.IOP.IOR ior1,
               org.omg.IOP.IOR ior2);

    //
    // IDL:ooc.com/OCI/ConFactory/hash:1.0
    //
    /**
     *
     * Calculates a hash value for an IOR.
     *
     * @param ref The IOR to calculate a hash value for.
     *
     * @param maximum The maximum value of the hash value.
     *
     * @return The hash value.
     *
     **/

    int
    hash(org.omg.IOP.IOR ref,
         int maximum);

    //
    // IDL:ooc.com/OCI/ConFactory/get_info:1.0
    //
    /**
     *
     * Returns the information object associated with the Connector
     * factory.
     *
     * @return The Connector factory information object.
     *
     **/

    ConFactoryInfo
    get_info();
}
