// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package test.poa;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;
import java.util.*;

final public class TestLocationForwardClient extends test.common.TestBase
{
    public static void
    main(String args[])
    {
        Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        ORB orb = null;

	try
	{
            //
            // Create ORB
            //
	    orb = ORB.init(args, props);

	    POA root = TestUtil.GetRootPOA(orb);

	    POA poa;
	    Policy[] policies;

	    POAManager manager = root.the_POAManager();

	    //
	    // Create POA
	    //
	    policies = new Policy[4];
	    policies[0] = root.create_id_assignment_policy(
		org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);
	    policies[1] = root.create_id_uniqueness_policy(
		org.omg.PortableServer.IdUniquenessPolicyValue.UNIQUE_ID);
	    policies[2] = root.create_servant_retention_policy(
		org.omg.PortableServer.ServantRetentionPolicyValue.RETAIN);
	    policies[3] = root.create_request_processing_policy(
		org.omg.PortableServer.RequestProcessingPolicyValue.
		USE_SERVANT_MANAGER);
	    try
	    {
		poa = root.create_POA("poa", manager, policies);
	    }
	    catch(InvalidPolicy ex)
	    {
		throw new RuntimeException();
	    }
	    catch(AdapterAlreadyExists ex)
	    {
		throw new RuntimeException();
	    }

	    TestLocationForwardActivator_impl activatorImpl =
		new TestLocationForwardActivator_impl();
	    ServantActivator activator = activatorImpl._this(orb);
	    try
	    {
		poa.set_servant_manager(activator);
	    }
	    catch(WrongPolicy ex)
	    {
		throw new RuntimeException();
	    }

	    byte[] oid = ("test").getBytes();
	    org.omg.CORBA.Object reference =
                poa.create_reference_with_id(oid, "IDL:Test:1.0");

	    //
	    // Read all object references from file
	    //
	    org.omg.CORBA.Object obj =
		orb.string_to_object("relfile:///Test.ref");
	    if(obj == null)
	    {
		System.err.println("TestLocationForwardClient: " +
				   "cannot read IOR from Test.ref");
		System.exit(1);
	    }
	    TestLocationForwardServer server =
		TestLocationForwardServerHelper.narrow(obj);
	    TEST(server != null);
	    org.omg.CORBA.Object servant = server.get_servant();

	    activatorImpl.setForwardRequest(servant);

	    TestLocationForward_impl testImpl =
		new TestLocationForward_impl(orb);
	    activatorImpl.setActivatedServant(testImpl);

	    try
	    {
		manager.activate();
	    }
	    catch( org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
	    {
		throw new RuntimeException();
	    }

	    server.setForwardRequest(reference);

	    //
	    // Run some calls
	    //
	    TestLocationForward local =
		TestLocationForwardHelper.narrow(reference);

	    //
	    // First should be local
	    //
	    local.aMethod();
	    local.deactivate_servant();

	    //
	    // Second, should be remote
	    //
	    local.aMethod();
	    local.deactivate_servant();

	    //
	    // Third should be local again
	    //
	    local.aMethod();
	    local.deactivate_servant();

	    //
	    // Clean up
	    //
	    poa.destroy(true, true);

	    server.deactivate();
	}
	catch(SystemException ex)
	{
            ex.printStackTrace();
            System.exit(1);
	}

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(SystemException ex)
            {
                ex.printStackTrace();
                System.exit(1);
            }
        }

	System.exit(0);
    }
}
