// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package test.poa;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;
import org.omg.PortableServer.POAManagerFactoryPackage.*;

import java.lang.System;

final class TestPOAManagerFactory extends test.common.TestBase
{
    static void
    TestAll(ORB orb, POA root)
    {
        POAManagerFactory factory = root.the_POAManagerFactory();
        com.ooc.OBPortableServer.POAManagerFactory pmFactory = 
            com.ooc.OBPortableServer.POAManagerFactoryHelper.narrow(factory);

        POAManager[] managers;

        //
        // Initial count: RootPOAManager
        //
        int count = 1;

        managers = pmFactory.list();
        TEST(managers.length == 1);
        
        //
        // Create POA Managers without policies
        //
        
        Policy[] pl = new Policy[0];;
        
        POAManager test = null;
        try
        {
            test = pmFactory.create_POAManager("TestPOAManager", pl);
            count++;
        }
        catch(ManagerAlreadyExists ex)
        {
            TEST(false);
        }
        catch(PolicyError ex)
        {
        }

        POAManager tmp = pmFactory.find("TestPOAManager");
        TEST(tmp == test);
        managers = pmFactory.list();

        //
        // count == 2
        //
        TEST(count == 2);
        TEST(managers.length == count);
        
        POAManager test2 = null;
        try
        {
            test2 = pmFactory.create_POAManager("TestPOAManager2", pl);
            count++;
        }
        catch(ManagerAlreadyExists ex)
        {
            TEST(false);
        }
        catch(PolicyError ex)
        {
        }

        tmp = pmFactory.find("TestPOAManager2");
        TEST(tmp == test2);
        managers = pmFactory.list();
        
        //
        // count == 3
        //
        TEST(count == 3);
        TEST(managers.length == count);

        try
        {
            test2 = pmFactory.create_POAManager("TestPOAManager2",pl);
            count++;

            //
            // 'create_POAManager' should not have succeeded
            //
            TEST(false);
        }
        catch(ManagerAlreadyExists ex)
        {
            //
            // Expected
            //
        }
        catch(PolicyError ex)
        {
        }

        TEST(tmp == test2);
        TEST(test != test2);

        managers = pmFactory.list();
        
        //
        // count == 3 (still)
        //
        TEST(count == 3);
        TEST(managers.length == count);

        //
        // Create POA Managers with proprietary policies
        //
        
        Policy[] policies = new Policy[2];
        boolean policiesOkay = true;
        try
        {
            policies[0] = pmFactory.create_communications_concurrency_policy(
                com.ooc.OBPortableServer.COMMUNICATIONS_CONCURRENCY_POLICY_THREADED.value);

            policies[1] = pmFactory.create_giop_version_policy(
                com.ooc.OBPortableServer.GIOP_VERSION_POLICY_1_2.value);
        }
        catch(PolicyError ex)
        {
            policiesOkay = false;
        }

        if(policiesOkay)
        {
            POAManager configManager = null;
            try
            {
                configManager = pmFactory.create_POAManager("ConfigManager", policies);
                count++;
            }
            catch(ManagerAlreadyExists ex)
            {
                TEST(false);
            }
            catch(PolicyError ex)
            {
            }

            tmp = pmFactory.find("ConfigManager");
            TEST(tmp == configManager);
        }
            
        managers = pmFactory.list();

        //
        // count == 4
        //
        TEST(count == 4);
        TEST(managers.length == count);

        //
        // Check the policy values
        //

        com.ooc.OBPortableServer.CommunicationsConcurrencyPolicy commsPolicy = 
            com.ooc.OBPortableServer.CommunicationsConcurrencyPolicyHelper.narrow(policies[0]);
        TEST(commsPolicy.value() == 
            com.ooc.OBPortableServer.COMMUNICATIONS_CONCURRENCY_POLICY_THREADED.value);
        TEST(commsPolicy.policy_type() ==
            com.ooc.OBPortableServer.COMMUNICATIONS_CONCURRENCY_POLICY_ID.value);

        com.ooc.OBPortableServer.GIOPVersionPolicy giopPolicy = 
            com.ooc.OBPortableServer.GIOPVersionPolicyHelper.narrow(policies[1]);
        TEST(giopPolicy.value() ==
            com.ooc.OBPortableServer.GIOP_VERSION_POLICY_1_2.value);
        TEST(giopPolicy.policy_type() ==
            com.ooc.OBPortableServer.GIOP_VERSION_POLICY_ID.value);
                
        //
        // EndpointConfigurationPolicy
        //
        String config = "iiop --host localhost --port 10999 --bind localhost";

        Policy[] policies2 = new Policy[1];
        policiesOkay = true;
        try
        {
            policies2[0] = pmFactory.create_endpoint_configuration_policy(config);
        }
        catch(PolicyError ex)
        {
            policiesOkay = false;
        }

        if(policiesOkay)
        {
            com.ooc.OBPortableServer.EndpointConfigurationPolicy endpointPolicy = 
                com.ooc.OBPortableServer.EndpointConfigurationPolicyHelper.narrow(policies2[0]);

            TEST(endpointPolicy.policy_type() == 
                com.ooc.OBPortableServer.ENDPOINT_CONFIGURATION_POLICY_ID.value);

            //
            // Create POA Manager with endpoint policy
            //
            POAManager endpointManager = null;
            try
            {
                endpointManager = pmFactory.create_POAManager("EndpointManager", policies2);
                count++;
            }
            catch(ManagerAlreadyExists ex)
            {
                TEST(false);
            }
            catch(PolicyError ex)
            {
            }

            tmp = pmFactory.find("EndpointManager");
            TEST(tmp == endpointManager);
        }

        managers = pmFactory.list();

        //
        // count == 5
        //
        TEST(count == 5);
        TEST(managers.length == count);
    }

    static void
    runtests(ORB orb, POA root)
    {
	TestAll(orb, root);
    }

    public static void
    main(String[] args)
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
        ORB orb = null;

	try
	{
            //
            // Create ORB
            //
	    orb = ORB.init(args, props);

	    POA root = TestUtil.GetRootPOA(orb);

	    System.out.print("Testing POAManagerFactory... ");
	    System.out.flush();

	    //
	    // Run the tests using the root POA
	    //
	    runtests(orb, root);

	    System.out.println("Done!");
	}
	catch(SystemException ex)
	{
            ex.printStackTrace();
	    status = 1;
	}

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(SystemException ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

	System.exit(status);
    }
}
