// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

// ----------------------------------------------------------------------
// DispatchThreadSameThread
// ----------------------------------------------------------------------

class DispatchSameThread_impl extends org.omg.CORBA.LocalObject
    implements DispatchStrategy
{
    DispatchSameThread_impl()
    {
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public int
    id()
    {
        return SAME_THREAD.value;
    }

    public org.omg.CORBA.Any
    info()
    {
        return new com.ooc.CORBA.Any();
    }

    public void
    dispatch(DispatchRequest request)
    {
        //
        // Invoke the request
        //
        request.invoke();
    }
}

// ----------------------------------------------------------------------
// DispatchThreadPerRequest
// ----------------------------------------------------------------------

class DispatchThreadPerRequest_impl extends org.omg.CORBA.LocalObject
    implements DispatchStrategy
{
    class Dispatcher extends Thread
    {
	private DispatchRequest request_;
    
	Dispatcher(DispatchRequest request)
	{
	    super("ORBacus:ThreadPerRequest:Dispatcher");
	    request_ = request;
	}

	public void
	run()
	{
            //
            // Invoke the request
            //
            request_.invoke();
	}
    }

    DispatchThreadPerRequest_impl()
    {
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public int
    id()
    {
        return THREAD_PER_REQUEST.value;
    }

    public org.omg.CORBA.Any
    info()
    {
        return new com.ooc.CORBA.Any();
    }

    public void
    dispatch(DispatchRequest request)
    {
	try
	{
	    Thread t = new Dispatcher(request);
	    t.start();
	}
	catch(OutOfMemoryError e)
	{
	    throw new org.omg.CORBA.TRANSIENT();
	}
    }
}

// ----------------------------------------------------------------------
// DispatchThreadPool_impl
// ----------------------------------------------------------------------

class DispatchThreadPool_impl extends org.omg.CORBA.LocalObject
    implements DispatchStrategy
{
    private int id_;
    private ThreadPool pool_;

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public int
    id()
    {
        return THREAD_POOL.value;
    }

    public org.omg.CORBA.Any
    info()
    {
        org.omg.CORBA.Any any = new com.ooc.CORBA.Any();
        any.insert_ulong(id_);
        return any;
    }

    public void
    dispatch(DispatchRequest request)
    {
	pool_.add(request);
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    DispatchThreadPool_impl(int id, ThreadPool pool)
    {
        id_ = id;
	pool_ = pool;
    }
}

public class DispatchStrategyFactory_impl extends org.omg.CORBA.LocalObject
    implements DispatchStrategyFactory
{
    //
    // A sequence of thread pools. The index in the sequence is the
    // thread pool id.
    //
    private java.util.Vector pools_ = new java.util.Vector();

    //
    // Has the default thread pool been created yet?
    //
    private boolean haveDefaultThreadPool_ = false;

    //
    // If so, what is the thread pool id?
    //
    private int defaultThreadPool_;

    //
    // Has the factory been destroyed?
    //
    private boolean destroy_ = false;

    //
    // The ORB instance
    //
    private ORBInstance orbInstance_ = null;

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public synchronized int
    create_thread_pool(int nthreads)
    {
	//
	// The ORB destroys this object, so it's an initialization
	// error if this operation is called after ORB destruction
	//
	if(destroy_)
	    throw new org.omg.CORBA.INITIALIZE(
	        MinorCodes.describeInitialize(MinorCodes.MinorORBDestroyed),
                MinorCodes.MinorORBDestroyed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	
	//
	// Find the first empty thread pool
	//
	int i;
	for(i = 0 ; i < pools_.size() ; i++)
	    if(pools_.elementAt(i) == null)
		break;

	//
	// If there is no empty slot then append an empty slot
	//
	if(i >= pools_.size())
	    pools_.addElement(null);

	//
	// Allocate a new ThreadPool
	//
	pools_.setElementAt(new ThreadPool(i, nthreads), i);

	return i;
    }

    public synchronized void
    destroy_thread_pool(int id)
	throws InvalidThreadPool
    {
	//
	// The ORB destroys this object, so it's an initialization error
	// if this operation is called after ORB destruction
	//
	if(destroy_)
	    throw new org.omg.CORBA.INITIALIZE(
	        MinorCodes.describeInitialize(MinorCodes.MinorORBDestroyed),
                MinorCodes.MinorORBDestroyed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	
	if(id < 0 || id > pools_.size() || pools_.elementAt(id) == null)
	    throw new InvalidThreadPool();
	
	//
	// Destroy the ThreadPool
	//
	((ThreadPool)pools_.elementAt(id)).destroy();
	
	//
	// Empty the slot associated with this thread pool
	//
	pools_.setElementAt(null, id);
    }

    public synchronized DispatchStrategy
    create_thread_pool_strategy(int id)
	throws InvalidThreadPool
    {
	//
	// The ORB destroys this object, so it's an initialization error
	// if this operation is called after ORB destruction
	//
	if(destroy_)
	    throw new org.omg.CORBA.INITIALIZE(
	        MinorCodes.describeInitialize(MinorCodes.MinorORBDestroyed),
                MinorCodes.MinorORBDestroyed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	
	if(id < 0 || id > pools_.size() || pools_.elementAt(id) == null)
	    throw new InvalidThreadPool();
	
	return new
            DispatchThreadPool_impl(id, (ThreadPool)pools_.elementAt(id));
    }

    public synchronized DispatchStrategy
    create_same_thread_strategy()
    {
	//
	// The ORB destroys this object, so it's an initialization error
	// if this operation is called after ORB destruction
	//
	if(destroy_)
	    throw new org.omg.CORBA.INITIALIZE(
	        MinorCodes.describeInitialize(MinorCodes.MinorORBDestroyed),
                MinorCodes.MinorORBDestroyed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	
        return new DispatchSameThread_impl();
    }
    
    public synchronized DispatchStrategy
    create_thread_per_request_strategy()
    {
	//
	// The ORB destroys this object, so it's an initialization error
	// if this operation is called after ORB destruction
	//
	if(destroy_)
	    throw new org.omg.CORBA.INITIALIZE(
	        MinorCodes.describeInitialize(MinorCodes.MinorORBDestroyed),
                MinorCodes.MinorORBDestroyed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        return new DispatchThreadPerRequest_impl();
    }

    public synchronized DispatchStrategy
    create_default_dispatch_strategy()
    {
	//
	// The ORB destroys this object, so it's an initialization error
	// if this operation is called after ORB destruction
	//
	if(destroy_)
	    throw new org.omg.CORBA.INITIALIZE(
	        MinorCodes.describeInitialize(MinorCodes.MinorORBDestroyed),
                MinorCodes.MinorORBDestroyed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	
        //
        // Get the ORB properties
        //
	java.util.Properties properties = orbInstance_.getProperties();

	//
	// Set the dispatch strategy policy as specified by the
	// conc_model property
	//
	String value = properties.getProperty("ooc.orb.oa.conc_model");

	if(value != null)
	{
	    if(value.equals("threaded") ||
	       value.equals("thread_per_client"))
	    {
		return create_same_thread_strategy();
	    }
	    else if(value.equals("thread_per_request"))
	    {
		return create_thread_per_request_strategy();
	    }
	    else if(value.equals("thread_pool"))
	    {
		//
		// If there is no default thread pool yet then create one,
		// with a default of 10 threads.
		//
		if(!haveDefaultThreadPool_)
		{
		    haveDefaultThreadPool_ = true;
		    value = properties.getProperty("ooc.orb.oa.thread_pool");
		    int nthreads = 0;
		    if(value != null)
			nthreads = Integer.parseInt(value);
		    if(nthreads == 0)
			nthreads = 10;
		    defaultThreadPool_ = create_thread_pool(nthreads);
                }
                try
                {
                    return create_thread_pool_strategy(defaultThreadPool_);
                }
                catch(InvalidThreadPool ex)
                {
                    Assert._OB_assert(false);
                }
	    }
	    else
	    {
		String err = "ooc.orb.oa.conc_model: Unknown value `";
		err += value;
		err += "'";
	        orbInstance_.getLogger().warning(err);
	    }
	}

	//
	// The default
	//
	return create_same_thread_strategy();
    }
    
    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    public
    DispatchStrategyFactory_impl()
    {
    }

    public synchronized void
    _OB_setORBInstance(ORBInstance orbInstance)
    {
	orbInstance_ = orbInstance;
    }

    protected synchronized void
    _OB_destroy()
    {
	//
	// The ORB destroys this object, so it's an initialization error
	// if this operation is called after ORB destruction
	//
	if(destroy_)
	    throw new org.omg.CORBA.INITIALIZE(
	        MinorCodes.describeInitialize(MinorCodes.MinorORBDestroyed),
                MinorCodes.MinorORBDestroyed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	
	destroy_ = true;
	
	orbInstance_ = null;
	
	//
	// Destroy each of the thread pools
	//
	for(int i = 0 ; i < pools_.size() ; i++)
	{
	    ThreadPool pool = (ThreadPool)pools_.elementAt(i);
	    if(pool != null)
	    {
		pool.destroy();
		pools_.setElementAt(null, i);
	    }
	}
    }
}
