// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OB;

public class FileURLScheme_impl extends org.omg.CORBA.LocalObject
                                implements URLScheme
{
    private boolean relative_;
    private URLRegistry registry_;

    // ------------------------------------------------------------------
    // FileURLScheme_impl constructor
    // ------------------------------------------------------------------

    public
    FileURLScheme_impl(boolean relative, URLRegistry registry)
    {
        relative_ = relative;
        registry_ = registry;
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    name()
    {
        return (relative_ ? "relfile" : "file");
    }

    public org.omg.CORBA.Object
    parse_url(String url)
    {
        int startIdx;
        if(relative_)
            startIdx = 8; // skip "relfile:"
        else
            startIdx = 5; // skip "file:"

        int len = url.length();

        //
        // Allow up to three leading '/''s to match commonly used forms
        // of the "file:" URL scheme
        // 
        for(int n = 0 ; startIdx < len && n < 3 ; n++, startIdx++)
            if(url.charAt(startIdx) != '/')
                break;

        if(startIdx >= len)
            throw new org.omg.CORBA.BAD_PARAM(
                MinorCodes.describeBadParam(
                    MinorCodes.MinorBadSchemeSpecificPart) +
                    ": no file name specified", 
                MinorCodes.MinorBadSchemeSpecificPart,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        String fileName;
        if(relative_)
            fileName = "";
        else
            fileName = "/";
        fileName += URLUtil.unescapeURL(url.substring(startIdx));

        try
        {
            java.io.FileInputStream file =
                new java.io.FileInputStream(fileName);
            java.io.BufferedReader in = new java.io.BufferedReader(
                new java.io.InputStreamReader(file));
            String ref = in.readLine();
            file.close();

            return registry_.parse_url(ref);
        }
        catch(java.io.IOException ex)
        {
	    System.err.println(ex);
            throw new org.omg.CORBA.BAD_PARAM(
                MinorCodes.describeBadParam(
                    MinorCodes.MinorBadSchemeSpecificPart) + ": file error",
                MinorCodes.MinorBadSchemeSpecificPart,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
    }

    public void
    destroy()
    {
        registry_ = null;
    }
}
