// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OBPortableServer;

//
// We don't need any sort of concurrency protection on this class
// since its lifecycle is tightly coupled with the POAManager. Once
// the POAManager destroys this class it should not be possible for
// any further requests to arrive (or any requests in the process of
// being dispatched).
//
final class POAOAInterface_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OB.OAInterface
{
    //
    // The ORBInstance
    //
    private com.ooc.OB.ORBInstance orbInstance_;

    //
    // The POAManager implementation
    //
    private POAManager_impl poaManager_;

    //
    // The boot manager implementation
    //
    private com.ooc.OB.BootManager_impl bootManagerImpl_;

    //
    // Is the POAOAInterface discarding requests?
    //
    private boolean discard_;

    // ----------------------------------------------------------------------
    // Package member implementation
    // ----------------------------------------------------------------------

    POAOAInterface_impl(POAManager_impl poaManager,
			com.ooc.OB.ORBInstance orbInstance)
    {
	poaManager_ = poaManager;
	orbInstance_ = orbInstance;
	bootManagerImpl_ =
            (com.ooc.OB.BootManager_impl)orbInstance.getBootManager();
    }

    // ----------------------------------------------------------------------
    // Public member implementation
    // ----------------------------------------------------------------------

    public com.ooc.OB.Upcall
    createUpcall(com.ooc.OB.UpcallReturn upcallReturn,
		 com.ooc.OCI.ProfileInfo profileInfo,
		 com.ooc.OCI.TransportInfo transportInfo,
		 int requestId,
                 String op,
		 com.ooc.CORBA.InputStream in,
		 org.omg.IOP.ServiceContext[] requestSCL)
    {
	com.ooc.OB.Upcall upcall = null;
	try
	{
	    //
	    // If discarding then throw a TRANSIENT exception
	    //
	    if(discard_)
		throw new org.omg.CORBA.TRANSIENT(
		    "Requests are being discarded",
		    0, org.omg.CORBA.CompletionStatus.COMPLETED_NO);

	    com.ooc.OB.ObjectKeyData data = new com.ooc.OB.ObjectKeyData();
	    if(com.ooc.OB.ObjectKey.ParseObjectKey(profileInfo.key, data))
	    {
		while(true)
		{
		    //
		    // Locate the POA. This may also throw a TRANSIENT
		    // exception if the POA manager is discarding.
		    //
		    org.omg.PortableServer.POA poa =
			poaManager_._OB_locatePOA(data);
		    if(poa != null)
		    {
			POA_impl poaImpl = (POA_impl)poa;
			upcall = poaImpl._OB_createUpcall(data.oid,
                                                          upcallReturn,
							  profileInfo,
							  transportInfo,
							  requestId,
							  op, in, requestSCL);
			//
			// If _OB_createUpcall returns a nil Upcall object
			// then we should retry since that means that the
			// POA is being destroyed
			//
			if(upcall == null)
			    continue;
		    }
		    break;
		}
	    }
	    else if(upcallReturn != null)
	    {
		//
		// Check to see if the BootManager knows of a reference
		// for the ObjectKey. If so, forward the request.
		//
		org.omg.IOP.IOR ior =
		    bootManagerImpl_._OB_locate(profileInfo.key);
		if(ior != null)
		    throw new com.ooc.OB.LocationForward(ior, false);
	    }
	    //
	    // If no upcall has been created then the object simply
	    // doesn't exist
	    //
	    if(upcall == null)
            {
                if(op.equals("_non_existent") || op.equals("_not_existent"))
                {
                    upcall = new com.ooc.OB.Upcall(orbInstance_, upcallReturn,
                                                   profileInfo, transportInfo,
                                                   requestId, op, in,
                                                   requestSCL);
                    upcall.preUnmarshal();
                    upcall.postUnmarshal();
                    upcall.postinvoke();
                    org.omg.CORBA.portable.OutputStream out =
                        upcall.preMarshal();
                    out.write_boolean(true);
                    upcall.postMarshal();
                }
                else
                    throw new org.omg.CORBA.OBJECT_NOT_EXIST(
                        com.ooc.OB.MinorCodes.describeObjectNotExist(
                            com.ooc.OB.MinorCodes.MinorCannotDispatch),
                        com.ooc.OB.MinorCodes.MinorCannotDispatch,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
	}
	catch(org.omg.CORBA.SystemException ex)
	{
	    upcall = new com.ooc.OB.Upcall(orbInstance_, upcallReturn,
					   profileInfo, transportInfo,
					   requestId, op, in, requestSCL);
	    upcall.setSystemException(ex);
        }
	catch(com.ooc.OB.LocationForward ex)
	{
	    upcall = new com.ooc.OB.Upcall(orbInstance_, upcallReturn,
					   profileInfo, transportInfo,
					   requestId, op, in, requestSCL);
	    upcall.setLocationForward(ex.ior, ex.perm);
	}

	com.ooc.OB.Assert._OB_assert(upcall != null);
	return upcall;
    }

    public int
    findByKey(byte[] key, org.omg.IOP.IORHolder ior)
    {
        com.ooc.OB.ObjectKeyData data = new com.ooc.OB.ObjectKeyData();
        if(com.ooc.OB.ObjectKey.ParseObjectKey(key, data))
        {
            try
            {
                org.omg.PortableServer.POA poa =
                    poaManager_._OB_locatePOA(data);
                if(poa != null)
                {
                    POA_impl poaImpl = (POA_impl)poa;
                    poaImpl._OB_locateServant(data.oid);
		    return com.ooc.OB.OAInterface.OBJECT_HERE;
                }
            }
            catch(org.omg.CORBA.SystemException ex)
            {
            }
            catch(com.ooc.OB.LocationForward fwd)
            {
                ior.value = fwd.ior;
                return (fwd.perm)
                    ? com.ooc.OB.OAInterface.OBJECT_FORWARD_PERM
                    : com.ooc.OB.OAInterface.OBJECT_FORWARD;
            }
        }
	else
	{
	    //
	    // Check to see if the BootManager knows of a reference
	    // for the ObjectKey. 
	    //
	    ior.value = bootManagerImpl_._OB_locate(key);
	    if(ior.value != null)
	        return com.ooc.OB.OAInterface.OBJECT_FORWARD;
	}
        return com.ooc.OB.OAInterface.UNKNOWN_OBJECT;
    }

    public com.ooc.OCI.ProfileInfo[]
    getUsableProfiles(org.omg.IOP.IOR ior,
                      org.omg.CORBA.Policy[] policies)
    {
	try
	{
	    com.ooc.OCI.Acceptor[] acceptors = poaManager_.get_acceptors();

            java.util.Vector seq = new java.util.Vector();
            for(int i = 0 ; i < acceptors.length ; i++)
            {
                com.ooc.OCI.ProfileInfo[] seq2 =
                    acceptors[i].get_local_profiles(ior);

                for(int j = 0 ; j < seq2.length ; j++)
                    seq.addElement(seq2[j]);
            }

            com.ooc.OCI.ProfileInfo[] result =
                new com.ooc.OCI.ProfileInfo[seq.size()];
            seq.copyInto(result);
            return result;
	}
	catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
	{
	    com.ooc.OB.Assert._OB_assert(false);
	    return null;
	}
    }

    //
    // Discard all incoming requests with a TRANSIENT exception
    //
    // ASYNC SAFE
    //
    public void
    discard()
    {
	discard_ = true;
    }

    //
    // Allow associated POAs to receive requests
    //
    public void
    activate()
    {
	discard_ = false;
    }
}
