// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OCI.IIOP;

import com.ooc.CORBA.InputStream;
import com.ooc.CORBA.OutputStream;

final class Acceptor_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.Acceptor
{
    // Some data members must not be private because the info object
    // must be able to access them
    public String[] hosts_;                     // The hosts
    public java.net.ServerSocket socket_;       // The socket
    private boolean multiProfile_;              // Use multiple profiles?
    private int port_;                          // The port
    private boolean keepAlive_;                 // The keepalive flag
    private java.net.InetAddress localAddress_; // The local address
    private AcceptorInfo_impl info_;            // Acceptor information
    private ListenerMap listenMap_;

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return PLUGIN_ID.value;
    }

    public int
    tag()
    {
	return org.omg.IOP.TAG_INTERNET_IOP.value;
    }

    public int
    handle()
    {
	throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    public void
    close()
    {
	//
	// Destroy the info object
	//
	info_._OB_destroy();

	//
	// Close the socket
	//
        try
        {
            socket_.close();
	    socket_ = null;
        }
        catch(java.io.IOException ex)
        {
        }
    }

    public void
    shutdown()
    {
        //
        // This operation does nothing in the java implementation
        //
    }

    public void
    listen()
    {
        //
        // This operation does nothing in the java implementation
        //
    }
    
    public com.ooc.OCI.Transport
    accept(boolean block)
    {
        //
        // A signed applet running in Netscape may be granted permission
        // to accept incoming connections. We use reflection to invoke
        // the Netscape PrivilegeManager class in an attempt to enable
        // the necessary privileges. If the applet hasn't been granted
        // these privileges, an exception will be raised. The privileges
        // are enabled only for the duration of this method.
        //
 	if(System.getProperty("java.vendor").indexOf("Netscape") != -1)
        {
 	    try
 	    {
 		Class nspm =
                    Class.forName("netscape.security.PrivilegeManager");
 		Class[] enablePrivilegeArgTypes = { String.class };
 		java.lang.reflect.Method enablePrivilege =
                    nspm.getMethod("enablePrivilege", enablePrivilegeArgTypes);
 		java.lang.Object[] enablePrivilegeArgs = { null };
 		String[] privileges =
                    { "IIOPRuntime", "UniversalAccept", "UniversalListen" };
 		for(int i = 0 ; i < privileges.length ; i++)
 		{
 		    enablePrivilegeArgs[0] = privileges[i];
 		    enablePrivilege.invoke(null, enablePrivilegeArgs);
 		}
 	    }
 	    catch(Exception ex)
            {
                // Ignore
            }
        }
    
	//
	// Accept
	//
	java.net.Socket socket;
        try
        {
            //
            // If non-blocking, use a timeout of 1ms
            //
            if(!block)
                socket_.setSoTimeout(1);
            else
                socket_.setSoTimeout(0);

	    socket = socket_.accept();
        }
        catch(java.io.InterruptedIOException ex)
        {
            if(!block)
                return null; // Timeout
            else
                throw new org.omg.CORBA.COMM_FAILURE(
                    com.ooc.OB.MinorCodes.describeCommFailure(
                        com.ooc.OB.MinorCodes.MinorAccept) +
                        ": " + ex.toString(),
                    com.ooc.OB.MinorCodes.MinorAccept,
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	}
        catch(java.io.IOException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorAccept) +
                    ": " + ex.toString(),
		com.ooc.OB.MinorCodes.MinorAccept,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	}

	//
	// Set TCP_NODELAY and SO_KEEPALIVE options
	//
	try
        {
            socket.setTcpNoDelay(true);
            if(keepAlive_)
                socket.setKeepAlive(true);
        }
        catch(java.net.SocketException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSetsockopt) +
                    ": " + ex.toString(),
		com.ooc.OB.MinorCodes.MinorSetsockopt,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

	//
	// Create new transport
	//
        com.ooc.OCI.Transport tr = null;
        try
        {
            tr = new Transport_impl(this, socket, listenMap_);
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            try
            {
                socket.close();
            }
            catch(java.io.IOException e)
            {
            }
            throw ex;
        }

	//
	// Call callbacks
	//
        com.ooc.OCI.TransportInfo trInfo = tr.get_info();
	try
	{
	    info_._OB_callAcceptCB(trInfo);
	}
	catch(org.omg.CORBA.SystemException ex)
	{
	    tr.close();
	    throw ex;
	}

	//
	// Return new transport
	//
	return tr;
    }

    public com.ooc.OCI.Transport
    connect_self()
    {
        //
        // Create socket and connect to local address
        //
        java.net.Socket socket = null;
        try
        {
            socket = new java.net.Socket(localAddress_, port_);
        }
        catch(java.net.ConnectException ex)
        {
            throw new org.omg.CORBA.TRANSIENT(
                com.ooc.OB.MinorCodes.describeTransient(
                    com.ooc.OB.MinorCodes.MinorConnectFailed) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorConnectFailed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch(java.io.IOException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSocket) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorSocket,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

	//
	// Set TCP_NODELAY option
	//
	try
        {
            socket.setTcpNoDelay(true);
        }
        catch(java.net.SocketException ex)
        {
	    try
	    {
		socket.close();
	    }
	    catch(java.io.IOException e)
	    {
	    }
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSetsockopt) +
                    ": " + ex.toString(),
		com.ooc.OB.MinorCodes.MinorSetsockopt,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

	//
	// Create and return new transport
	//
        com.ooc.OCI.Transport tr = null;
        try
        {
            tr = new Transport_impl(this, socket, listenMap_);
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            try
            {
                socket.close();
            }
            catch(java.io.IOException e)
            {
            }
            throw ex;
        }
	return tr;
    }

    public void
    add_profiles(com.ooc.OCI.ProfileInfo profileInfo,
                 org.omg.IOP.IORHolder ior)
    {
        if(port_ == 0)
            throw new RuntimeException();

        //
        // Filter components according to IIOP version
        //
        java.util.Vector components = new java.util.Vector();
        if(profileInfo.major == 1 && profileInfo.minor == 0)
        {
            //
            // No components for IIOP 1.0
            //
        }
        else
        {
            for(int i = 0 ; i < profileInfo.components.length ; i++)
                components.addElement(profileInfo.components[i]);
        }

        if(profileInfo.major == 1 && profileInfo.minor == 0)
        {
            //
            // For IIOP 1.0, we always add one profile for each host,
            // since IIOP 1.0 doesn't support tagged components in a
            // profile
            //
            for(int i = 0 ; i < hosts_.length ; i++)
            {
                org.omg.IIOP.ProfileBody_1_0 body =
                    new org.omg.IIOP.ProfileBody_1_0();
                body.iiop_version =
                    new org.omg.IIOP.Version(profileInfo.major,
                                             profileInfo.minor);
                body.host = hosts_[i];
                if(port_ >= 0x8000)
                    body.port = (short)(port_ - 0xffff - 1);
                else
                    body.port = (short)port_;
                body.object_key = profileInfo.key;

                int len = ior.value.profiles.length + 1;
                org.omg.IOP.TaggedProfile[] profiles =
                    new org.omg.IOP.TaggedProfile[len];
                System.arraycopy(ior.value.profiles, 0, profiles, 0,
                                 ior.value.profiles.length);
                ior.value.profiles = profiles;
                ior.value.profiles[len - 1] = new org.omg.IOP.TaggedProfile();
                ior.value.profiles[len - 1].tag =
                    org.omg.IOP.TAG_INTERNET_IOP.value;
                com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer();
                OutputStream out = new OutputStream(buf);
                out._OB_writeEndian();
                org.omg.IIOP.ProfileBody_1_0Helper.write(out, body);
                ior.value.profiles[len - 1].profile_data =
                    new byte[buf.length()];
                System.arraycopy(buf.data(), 0,
                                 ior.value.profiles[len - 1].profile_data,
                                 0, buf.length());
            }
        }
        else
        {
            if(multiProfile_)
            {
                //
                // Add one profile for each host
                //

                for(int i = 0 ; i < hosts_.length ; i++)
                {
                    org.omg.IIOP.ProfileBody_1_1 body =
                        new org.omg.IIOP.ProfileBody_1_1();
                    body.iiop_version =
                        new org.omg.IIOP.Version(profileInfo.major,
                                                 profileInfo.minor);
                    body.host = hosts_[i];
                    if(port_ >= 0x8000)
                        body.port = (short)(port_ - 0xffff - 1);
                    else
                        body.port = (short)port_;
                    body.object_key = profileInfo.key;
                    body.components =
                        new org.omg.IOP.TaggedComponent[components.size()];
                    components.copyInto(body.components);

                    int len = ior.value.profiles.length + 1;
                    org.omg.IOP.TaggedProfile[] profiles =
                        new org.omg.IOP.TaggedProfile[len];
                    System.arraycopy(ior.value.profiles, 0, profiles, 0,
                                     ior.value.profiles.length);
                    ior.value.profiles = profiles;
                    ior.value.profiles[len - 1] =
                        new org.omg.IOP.TaggedProfile();
                    ior.value.profiles[len - 1].tag =
                        org.omg.IOP.TAG_INTERNET_IOP.value;
                    com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer();
                    OutputStream out = new OutputStream(buf);
                    out._OB_writeEndian();
                    org.omg.IIOP.ProfileBody_1_1Helper.write(out, body);
                    ior.value.profiles[len - 1].profile_data =
                        new byte[buf.length()];
                    System.arraycopy(buf.data(), 0,
                                     ior.value.profiles[len - 1].profile_data,
                                     0, buf.length());
                }
            }
            else
            {
                //
                // Add a single tagged profile. If there are additional
                // hosts, add a tagged component for each host.
                //

                org.omg.IIOP.ProfileBody_1_1 body =
                    new org.omg.IIOP.ProfileBody_1_1();
                body.iiop_version =
                    new org.omg.IIOP.Version(profileInfo.major,
                                             profileInfo.minor);
                body.host = hosts_[0];
                if(port_ >= 0x8000)
                    body.port = (short)(port_ - 0xffff - 1);
                else
                    body.port = (short)port_;
                body.object_key = profileInfo.key;

                for(int i = 1 ; i < hosts_.length ; i++)
                {
                    org.omg.IOP.TaggedComponent c =
                        new org.omg.IOP.TaggedComponent();
                    c.tag = org.omg.IOP.TAG_ALTERNATE_IIOP_ADDRESS.value;
                    com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer();
                    OutputStream out = new OutputStream(buf);
                    out._OB_writeEndian();
                    out.write_string(hosts_[i]);
                    out.write_ushort(body.port);
                    c.component_data = new byte[buf.length()];
                    System.arraycopy(buf.data(), 0, c.component_data, 0,
                                     buf.length());
                    components.addElement(c);
                }
                body.components =
                    new org.omg.IOP.TaggedComponent[components.size()];
                components.copyInto(body.components);

                int len = ior.value.profiles.length + 1;
                org.omg.IOP.TaggedProfile[] profiles =
                    new org.omg.IOP.TaggedProfile[len];
                System.arraycopy(ior.value.profiles, 0, profiles, 0,
                                 ior.value.profiles.length);
                ior.value.profiles = profiles;
                ior.value.profiles[len - 1] = new org.omg.IOP.TaggedProfile();
                ior.value.profiles[len - 1].tag =
                    org.omg.IOP.TAG_INTERNET_IOP.value;
                com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer();
                OutputStream out = new OutputStream(buf);
                out._OB_writeEndian();
                org.omg.IIOP.ProfileBody_1_1Helper.write(out, body);
                ior.value.profiles[len - 1].profile_data =
                    new byte[buf.length()];
                System.arraycopy(buf.data(), 0,
                                 ior.value.profiles[len - 1].profile_data,
                                 0, buf.length());
            }
        }
    }

    public com.ooc.OCI.ProfileInfo[]
    get_local_profiles(org.omg.IOP.IOR ior)
    {
        //
        // Get local profiles for all hosts
        //
        com.ooc.OCI.ProfileInfoSeqHolder profileInfoSeq =
            new com.ooc.OCI.ProfileInfoSeqHolder();
        profileInfoSeq.value = new com.ooc.OCI.ProfileInfo[0];

        for(int i = 0 ; i < hosts_.length ; i++)
            Util.extractAllProfileInfos(ior, profileInfoSeq, true, hosts_[i],
                                        port_, true);

        return profileInfoSeq.value;
    }

    public com.ooc.OCI.AcceptorInfo
    get_info()
    {
        return info_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    public
    Acceptor_impl(String[] hosts, boolean multiProfile, 
                  int port, int backlog, boolean keepAlive,
                  ListenerMap lm)
    {
	//System.out.println("Acceptor_impl");

        hosts_ = hosts;
        multiProfile_ = multiProfile;
        keepAlive_ = keepAlive;
        info_ = new AcceptorInfo_impl(this);
        listenMap_ = lm;

        if(backlog == 0)
            backlog = 50; // 50 is the JDK's default value

        //
        // Get the local address for use by connect_self. Since we are
        // binding to all network interfaces, we'll use the loopback
        // address.
        //
        try
        {
            localAddress_ = java.net.InetAddress.getLocalHost();
        }
        catch(java.net.UnknownHostException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorGethostbyname) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorGethostbyname,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        //
        // Create socket and bind to all network interfaces
        //
        try
        {
            socket_ = new java.net.ServerSocket(port, backlog);

            //
            // Read back the port. This is needed if the port was selected by
            // the operating system.
            //
	    port_ = socket_.getLocalPort();
        }
        catch(java.net.BindException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorBind) +
                    ": " + ex.toString(),
		com.ooc.OB.MinorCodes.MinorBind,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch(java.io.IOException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSocket) +
                    ": " + ex.toString(),
		com.ooc.OB.MinorCodes.MinorSocket,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        // 
        // add these endpoints to the listenMap_
        //
        synchronized(listenMap_)
        {
            for(int i = 0; i < hosts_.length; i++)
                listenMap_.add(hosts_[i], (short)port_);
        }
    }

    public
    Acceptor_impl(String address, String[] hosts, boolean multiProfile,
                  int port, int backlog, boolean keepAlive,
                  ListenerMap lm)
    {
        //System.out.println("Acceptor_impl");
        hosts_ = hosts;
        multiProfile_ = multiProfile;
        keepAlive_ = keepAlive;
        info_ = new AcceptorInfo_impl(this);
        listenMap_ = lm;

        if(backlog == 0)
            backlog = 50; // 50 is the JDK's default value

        //
        // Get the local address for use by connect_self
        //
        try
        {
            localAddress_ = java.net.InetAddress.getByName(address);
        }
        catch(java.net.UnknownHostException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorGethostbyname) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorGethostbyname,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        //
        // Create socket and bind to requested network interface
        //
        try
        {
            socket_ = new java.net.ServerSocket(port, backlog, localAddress_);

            //
            // Read back the port. This is needed if the port was selected by
            // the operating system.
            //
            port_ = socket_.getLocalPort();
        }
        catch(java.net.BindException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorBind) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorBind,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch(java.io.IOException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSocket) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorSocket,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        //
        // Add this entry to the listenMap_ as an endpoint to remap
        //
        synchronized(listenMap_)
        {
            for(int i = 0; i < hosts_.length; i++)
                listenMap_.add(hosts_[i], (short)port_);
        }
    }

    public void
    finalize()
	throws Throwable
    {
	//System.out.println("~Acceptor_impl");
	if(socket_ != null)
	    close();

        //
        // remove this acceptor from the listenMap_
        //
        synchronized(listenMap_)
        {
            for(int i = 0; i < hosts_.length; i++)
                listenMap_.remove(hosts_[i], (short)port_);
        }

	super.finalize();
    }
}
