// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OCI.IIOP;

final class ConFactory_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.ConFactory
{
    private boolean keepAlive_;         // The keepalive flag
    private ConFactoryInfo_impl info_;  // ConFactory info
    private ListenerMap listenMap_;

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return PLUGIN_ID.value;
    }

    public int
    tag()
    {
        return org.omg.IOP.TAG_INTERNET_IOP.value;
    }

    public String
    describe_profile(org.omg.IOP.TaggedProfile profile)
    {
        com.ooc.OB.Assert._OB_assert(profile.tag ==
                                 org.omg.IOP.TAG_INTERNET_IOP.value);

        //
        // Get the IIOP profile body
        //
        byte[] data = profile.profile_data;
        com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer(data, data.length);
        com.ooc.CORBA.InputStream in = new com.ooc.CORBA.InputStream(buf);
        in._OB_readEndian();
        org.omg.IIOP.ProfileBody_1_0 body =
            org.omg.IIOP.ProfileBody_1_0Helper.read(in);

        StringBuffer result = new StringBuffer();

        //
        // Show general info
        //
        result.append("iiop_version: " + (int)body.iiop_version.major + '.'
            + (int)body.iiop_version.minor + '\n');
        result.append("host: " + body.host + '\n');
        int port;
        if(body.port < 0)
            port = 0xffff + (int)body.port + 1;
        else
            port = body.port;
        result.append("port: " + port + '\n');
        result.append("object_key: (" + body.object_key.length + ")\n");
        String tmp = com.ooc.OB.IORUtil.dump_octets(body.object_key, 0,
                                                    body.object_key.length);
        result.append(tmp);

        //
        // Print IIOP 1.1 information (components)
        //
        if(body.iiop_version.major > 1 || body.iiop_version.minor >= 1)
        {
            int l = in.read_ulong();

            for(int i = 0 ; i < l ; i++)
            {
                org.omg.IOP.TaggedComponent component =
                    org.omg.IOP.TaggedComponentHelper.read(in);

                String desc = com.ooc.OB.IORUtil.describe_component(component);
                result.append(desc);
            }
        }

        return result.toString();
    }

    public com.ooc.OCI.Connector[]
    create_connectors(org.omg.IOP.IOR ior, org.omg.CORBA.Policy[] policies)
    {
        //
        // Check whether policies are satisfied
        //
        for(int i = 0; i < policies.length; i++)
        {
            if(policies[i].policy_type() ==
               com.ooc.OB.PROTOCOL_POLICY_ID.value)
            {
                com.ooc.OB.ProtocolPolicy protocolPolicy =
                    com.ooc.OB.ProtocolPolicyHelper.narrow(policies[i]);
                if(!protocolPolicy.contains(PLUGIN_ID.value))
                    return new com.ooc.OCI.Connector[0];
            }
        }

        //
        // Create Connectors from profiles
        //
        java.util.Vector seq = new java.util.Vector();
	for(int i = 0 ; i < ior.profiles.length ; i++)
	{
	    if(ior.profiles[i].tag == tag())
	    {
		//
		// Get the IIOP profile body
		//
		byte[] data = ior.profiles[i].profile_data;
                com.ooc.OCI.Buffer buf =
                    new com.ooc.OCI.Buffer(data, data.length);
		com.ooc.CORBA.InputStream in =
		    new com.ooc.CORBA.InputStream(buf, 0, false);
		in._OB_readEndian();
		org.omg.IIOP.ProfileBody_1_0 body =
		    org.omg.IIOP.ProfileBody_1_0Helper.read(in);

		//
		// Create new connector for this profile
		//
		int port;
		if(body.port < 0)
		    port = 0xffff + (int)body.port + 1;
		else
		    port = (int)body.port;
                com.ooc.OCI.ConnectCB[] cbs = info_._OB_getConnectCBSeq();
		seq.addElement(new Connector_impl(body.host, port, 
                                                  keepAlive_, cbs,
                                                  listenMap_));

                //
                // If this is a 1.1 profile, check for
                // TAG_ALTERNATE_IIOP_ADDRESS in the components
                //
                if(body.iiop_version.major > 1 || body.iiop_version.minor > 0)
                {
                    //
                    // Unmarshal the tagged components
                    //
                    int len = in.read_ulong();
                    org.omg.IOP.TaggedComponent[] components =
                        new org.omg.IOP.TaggedComponent[len];
                    for(int c = 0 ; c < len ; c++)
                        components[c] =
                            org.omg.IOP.TaggedComponentHelper.read(in);

                    //
                    // Check for TAG_ALTERNATE_IIOP_ADDRESS
                    //
                    for(int c = 0 ; c < components.length ; c++)
                        if(components[c].tag ==
                           org.omg.IOP.TAG_ALTERNATE_IIOP_ADDRESS.value)
                        {
                            byte[] cdata = components[c].component_data;
                            int clen = components[c].component_data.length;
                            com.ooc.OCI.Buffer cbuf =
                                new com.ooc.OCI.Buffer(cdata, clen);
                            com.ooc.CORBA.InputStream cin =
                                new com.ooc.CORBA.InputStream(cbuf, 0, false);
                            cin._OB_readEndian();
                            String host = cin.read_string();
                            short s = cin.read_ushort();
                            int cport;
                            if(s < 0)
                                cport = 0xffff + (int)s + 1;
                            else
                                cport = (int)s;

                            //
                            // Create new connector for this component
                            //
                            com.ooc.OCI.ConnectCB[] ccbs =
                                info_._OB_getConnectCBSeq();
                            seq.addElement(
                                new Connector_impl(host, cport, 
                                                   keepAlive_, ccbs,
                                                   listenMap_));
                        }
                }
	    }
	}

        com.ooc.OCI.Connector[] result = new com.ooc.OCI.Connector[seq.size()];
        seq.copyInto(result);
        return result;
    }

    public boolean
    equivalent(org.omg.IOP.IOR ior1, org.omg.IOP.IOR ior2)
    {
	return Util.equivalent(ior1, ior2);
    }

    public int
    hash(org.omg.IOP.IOR ior, int max)
    {
        return Util.hash(ior, max);
    }
    
    public com.ooc.OCI.ConFactoryInfo
    get_info()
    {
        return info_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    public
    ConFactory_impl(boolean keepAlive,
                    ListenerMap lm)
    {
	//System.out.println("ConFactory");
        keepAlive_ = keepAlive;
        info_ = new ConFactoryInfo_impl();
        listenMap_ = lm;
    }

    public void
    finalize()
	throws Throwable
    {
	//System.out.println("~ConFactory");
	super.finalize();
    }
}
