// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OCI.IIOP;

final class Connector_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.Connector
{
    // Some data member must not be private because the info object
    // must be able to access them
    public String host_;                // The host
    public int port_;                   // The port
    private boolean keepAlive_;         // The keepalive flag
    private ConnectorInfo_impl info_;   // Connector information
    private java.net.Socket socket_;    // The socket
    private ListenerMap listenMap_;

    // ------------------------------------------------------------------
    // Private and protected functions
    // ------------------------------------------------------------------

    private void
    close()
    {
	//
	// Destroy the info object
	//
	info_._OB_destroy();

	//
	// Close the socket
	//
        try
        {
            socket_.close();
	    socket_ = null;
        }
        catch(java.io.IOException ex)
        {
        }
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return PLUGIN_ID.value;
    }

    public int
    tag()
    {
	return org.omg.IOP.TAG_INTERNET_IOP.value;
    }

    public com.ooc.OCI.Transport
    connect()
    {
	if(socket_ != null)
	    close();

        //
        // Get the address
        //
        java.net.InetAddress address = null;
        try
        {
            address = java.net.InetAddress.getByName(host_);
        }
        catch(java.net.UnknownHostException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorGethostbyname) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorGethostbyname,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        //
        // A signed applet running in Netscape may be granted permission
        // to make unrestricted connections. We use reflection to invoke
        // the Netscape PrivilegeManager class in an attempt to enable
        // the necessary privileges. If the applet hasn't been granted
        // these privileges, an exception will be raised. The privileges
        // are enabled only for the duration of this method.
        //
        if(System.getProperty("java.vendor").indexOf("Netscape") != -1)
        {
            try
            {
                Class nspm =
                    Class.forName("netscape.security.PrivilegeManager");
                Class[] enablePrivilegeArgTypes = { String.class };
                java.lang.reflect.Method enablePrivilege =
                    nspm.getMethod("enablePrivilege", enablePrivilegeArgTypes);
                java.lang.Object[] enablePrivilegeArgs = { null };
                String[] privileges = { "IIOPRuntime", "UniversalConnect" };
                for(int i = 0 ; i < privileges.length ; i++)
                {
                    enablePrivilegeArgs[0] = privileges[i];
                    enablePrivilege.invoke(null, enablePrivilegeArgs);
                }
            }
            catch(Exception ex)
            {
                // Ignore
            }
        }

        //
        // Create socket and connect
        //
        try
        {
            socket_ = new java.net.Socket(address, port_);
        }
        catch(java.net.ConnectException ex)
        {
            throw new org.omg.CORBA.TRANSIENT(
                com.ooc.OB.MinorCodes.describeTransient(
                    com.ooc.OB.MinorCodes.MinorConnectFailed) +
                    ": " + ex.toString(),
		com.ooc.OB.MinorCodes.MinorConnectFailed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch(java.io.IOException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSocket) +
                    ": " + ex.toString(),
		com.ooc.OB.MinorCodes.MinorSocket,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

	//
	// Set TCP_NODELAY and SO_KEEPALIVE options
	//
	try
        {
            socket_.setTcpNoDelay(true);

	    if(keepAlive_)
		socket_.setKeepAlive(true);
        }
        catch(java.net.SocketException ex)
        {
	    try
	    {
		socket_.close();
	    }
	    catch(java.io.IOException e)
	    {
	    }
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSetsockopt) +
                    ": " + ex.toString(),
		com.ooc.OB.MinorCodes.MinorSetsockopt,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

	//
	// Create new transport
	//
        com.ooc.OCI.Transport tr = null;
        try
        {
            tr = new Transport_impl(this, socket_, listenMap_);
            socket_ = null;
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            try
            {
                socket_.close();
            }
            catch(java.io.IOException e)
            {
            }
            throw ex;
        }

	//
	// Call callbacks
	//
        com.ooc.OCI.TransportInfo trInfo = tr.get_info();
	try
	{
	    info_._OB_callConnectCB(trInfo);
	}
	catch(org.omg.CORBA.SystemException ex)
	{
	    tr.close();
	    throw ex;
	}

	//
	// Return new transport
	//
	return tr;
    }

    //
    // Helper class for connect_timeout()
    //
    private class ConnectTimeout extends Thread
    {
        private java.net.InetAddress address_ = null;
        private java.net.Socket so_ = null;
        private java.io.IOException ex_ = null;
        private boolean finished_ = false;
        private boolean timeout_ = false;

        ConnectTimeout(java.net.InetAddress address)
        {
            address_ = address;
        }

        public void
        run()
        {
            try
            {
                so_ = new java.net.Socket(address_, port_);
            }
            catch(java.io.IOException ex)
            {
                ex_ = ex;
            }

            synchronized(this)
            {
                if(timeout_)
                {
                    if(so_ != null)
                    {
                        try
                        {
                            so_.close();
                        }
                        catch(java.io.IOException ex)
                        {
                        }

                        so_ = null;
                    }
                }
                else
                {
                    finished_ = true;
                    ConnectTimeout.this.notify();
                }
            }
        }

        synchronized java.net.Socket
        waitForConnect(int t)
            throws java.io.IOException
        {
            while(!finished_)
            {
                try
                {
                    ConnectTimeout.this.wait(t);
                }
                catch(InterruptedException ex)
                {
                    continue;
                }

                if(!finished_) // Timeout
                {
                    timeout_ = true;
                    return null;
                }
            }

            if(so_ != null) // Connect succeeded
                return so_;

            if(ex_ != null) // Connect failed
                throw ex_;

            throw new InternalError();
        }
    }

    public com.ooc.OCI.Transport
    connect_timeout(int t)
    {
	if(socket_ != null)
	    close();

        //
        // Get the address
        //
        java.net.InetAddress address = null;
        try
        {
            address = java.net.InetAddress.getByName(host_);
        }
        catch(java.net.UnknownHostException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorGethostbyname) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorGethostbyname,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        //
        // Create socket and connect
        //
        try
        {
            ConnectTimeout connectTimeout = new ConnectTimeout(address);
            connectTimeout.start();

            socket_ = connectTimeout.waitForConnect(t);

            if(socket_ == null)
                return null;
        }
        catch(java.net.ConnectException ex)
        {
            throw new org.omg.CORBA.TRANSIENT(
                com.ooc.OB.MinorCodes.describeTransient(
                    com.ooc.OB.MinorCodes.MinorConnectFailed) +
                    ": " + ex.toString(),
		com.ooc.OB.MinorCodes.MinorConnectFailed,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch(java.io.IOException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSocket) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorSocket,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

	//
	// Set TCP_NODELAY and SO_KEEPALIVE options
	//
	try
        {
            socket_.setTcpNoDelay(true);

	    if(keepAlive_)
		socket_.setKeepAlive(true);
         }
        catch(java.net.SocketException ex)
        {
	    try
	    {
		socket_.close();
	    }
	    catch(java.io.IOException e)
	    {
	    }
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSetsockopt) +
                    ": " + ex.toString(),
		com.ooc.OB.MinorCodes.MinorSetsockopt,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

	//
	// Create new transport
	//
        com.ooc.OCI.Transport tr = null;
        try
        {
            tr = new Transport_impl(this, socket_, listenMap_);
            socket_ = null;
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            try
            {
                socket_.close();
            }
            catch(java.io.IOException e)
            {
            }
            throw ex;
        }

	//
	// Call callbacks
	//
        com.ooc.OCI.TransportInfo trInfo = tr.get_info();
	try
	{
	    info_._OB_callConnectCB(trInfo);
	}
	catch(org.omg.CORBA.SystemException ex)
	{
	    tr.close();
	    throw ex;
	}

	//
	// Return new transport
	//
	return tr;
    }

    public com.ooc.OCI.ProfileInfo[]
    get_usable_profiles(org.omg.IOP.IOR ior, org.omg.CORBA.Policy[] policies)
    {
        //
        // Make sure that the set of policies is met
        //
        for(int i = 0 ; i < policies.length ; i++)
        {
            if(policies[i].policy_type() ==
               com.ooc.OB.PROTOCOL_POLICY_ID.value)
            {
                com.ooc.OB.ProtocolPolicy protocolPolicy =
                    com.ooc.OB.ProtocolPolicyHelper.narrow(policies[i]);
                if(!protocolPolicy.contains(PLUGIN_ID.value))
                    return new com.ooc.OCI.ProfileInfo[0];
            }
        }

        com.ooc.OCI.ProfileInfoSeqHolder profileInfoSeq =
            new com.ooc.OCI.ProfileInfoSeqHolder();
        profileInfoSeq.value = new com.ooc.OCI.ProfileInfo[0];
        Util.extractAllProfileInfos(ior, profileInfoSeq, true, host_, port_,
                                    false);
        return profileInfoSeq.value;
    }

    public boolean
    equal(com.ooc.OCI.Connector con)
    {
        Connector_impl impl = null;
        try
        {
            impl = (Connector_impl)con;
        }
        catch(ClassCastException ex)
        {
            return false;
        }

        //
        // Compare ports
        //
        if(port_ != impl.port_)
            return false;

        //
        // Direct host name comparison
        //
        if(!host_.equals(impl.host_))
        {
            //
            // Direct host name comparision failed - must look up
            // addresses to be really sure if the hosts differ
            //
            try
            {
                java.net.InetAddress addr1 =
                    java.net.InetAddress.getByName(host_);

                java.net.InetAddress addr2 =
                    java.net.InetAddress.getByName(impl.host_);

                if(!addr1.equals(addr2))
                    return false;
            }
            catch(java.net.UnknownHostException ex)
            {
                //
                // Return false on hostname lookup failure
                //
                return false;
            }
        }

        //
        // OK, connectors are the same
        //
        return true;
    }

    public com.ooc.OCI.ConnectorInfo
    get_info()
    {
        return info_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    public
    Connector_impl(String host, int port, boolean keepAlive,
                   com.ooc.OCI.ConnectCB[] cb,
                   ListenerMap lm)
    {
	//System.out.println("Connector_impl");
	host_ = host;
	port_ = port;
        keepAlive_ = keepAlive;
        info_ = new ConnectorInfo_impl(this, cb);
        listenMap_ = lm;
    }

    public void
    finalize()
	throws Throwable
    {
	//System.out.println("~Connector_impl");
	if(socket_ != null)
	    close();

	super.finalize();
    }
}
