// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OCI.IIOP;

public final class TransportInfo_impl 
    extends org.omg.CORBA.LocalObject
    implements TransportInfo
{
    private com.ooc.OCI.ConnectorInfo connectorInfo_;   // connector info
    private com.ooc.OCI.AcceptorInfo acceptorInfo_;     // acceptor info
    private Transport_impl transport_;                  // associated transport
    private org.omg.IIOP.ListenPoint[] listenPoints_ = null;
    private ListenerMap listenMap_;

    //
    // All close callback objects
    //
    private java.util.Vector closeCBVec_ = new java.util.Vector();

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return PLUGIN_ID.value;
    }

    public int
    tag()
    {
        return org.omg.IOP.TAG_INTERNET_IOP.value;
    }

    public short
    origin()
    {
	if(acceptorInfo_ == null)
	    return com.ooc.OCI.CLIENT_SIDE.value;
	else
	    return com.ooc.OCI.SERVER_SIDE.value;
    }

    public synchronized String
    describe()
    {
        String desc = "id: " + PLUGIN_ID.value;

        String localAddr = addr();
        short localPort = port();
        desc += "\nlocal address: ";
        desc += localAddr;
        desc += ":";
        desc += (localPort < 0 ? 0xffff + (int)localPort + 1 : localPort);

        String remoteAddr = remote_addr();
        short remotePort = remote_port();
        desc += "\nremote address: ";
        desc += remoteAddr;
        desc += ":";
        desc += (remotePort < 0 ? 0xffff + (int)remotePort + 1 : remotePort);

        return desc;
    }

    public synchronized com.ooc.OCI.ConnectorInfo
    connector_info()
    {
        return connectorInfo_;
    }
    
    public synchronized com.ooc.OCI.AcceptorInfo
    acceptor_info()
    {
        return acceptorInfo_;
    }

    public synchronized void
    add_close_cb(com.ooc.OCI.CloseCB cb)
    {
        int length = closeCBVec_.size();
        for(int i = 0 ; i < length ; i++)
            if(closeCBVec_.elementAt(i) == cb)
                return; // Already registered
        closeCBVec_.addElement(cb);
    }

    public synchronized void
    remove_close_cb(com.ooc.OCI.CloseCB cb)
    {
        int length = closeCBVec_.size();
        for(int i = 0 ; i < length ; i++)
            if(closeCBVec_.elementAt(i) == cb)
            {
                closeCBVec_.removeElementAt(i);
                return;
            }
    }

    public synchronized String
    addr()
    {
	if(transport_ == null)
	    throw new org.omg.CORBA.NO_RESOURCES();

        return transport_.socket_.getLocalAddress().getHostAddress();
    }

    public synchronized short
    port()
    {
	if(transport_ == null)
	    throw new org.omg.CORBA.NO_RESOURCES();

        int port = transport_.socket_.getLocalPort();

	if(port >= 0x8000)
	    return (short)(port - 0xffff - 1);
	else
	    return (short)port;
    }

    public synchronized String
    remote_addr()
    {
	if(transport_ == null)
	    throw new org.omg.CORBA.NO_RESOURCES();

        return transport_.socket_.getInetAddress().getHostAddress();
    }

    public synchronized short
    remote_port()
    {
	if(transport_ == null)
	    throw new org.omg.CORBA.NO_RESOURCES();

        int port = transport_.socket_.getPort();

	if(port >= 0x8000)
	    return (short)(port - 0xffff - 1);
	else
	    return (short)port;
    }
    
    public org.omg.IOP.ServiceContext[]
    get_service_contexts(org.omg.CORBA.Policy[] policies)
    {
        org.omg.IOP.ServiceContext[] scl;
        boolean bHaveBidir = false;
        
        for(int i = 0; i < policies.length; i++)
        {
            if(policies[i].policy_type() == 
                    org.omg.BiDirPolicy.BIDIRECTIONAL_POLICY_TYPE.value)
            {
                org.omg.BiDirPolicy.BidirectionalPolicy p =
                    org.omg.BiDirPolicy.BidirectionalPolicyHelper.
                    narrow(policies[i]);
                if(p.value() == org.omg.BiDirPolicy.BOTH.value)
                    bHaveBidir = true;
                break;
            }
        }
        
        if(bHaveBidir)
        {
            org.omg.IIOP.BiDirIIOPServiceContext biDirCtxt = 
                new org.omg.IIOP.BiDirIIOPServiceContext();
            biDirCtxt.listen_points = listenMap_.getListenPoints();

            com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer();
            com.ooc.CORBA.OutputStream out = 
                new com.ooc.CORBA.OutputStream(buf);
            
            out._OB_writeEndian();
            org.omg.IIOP.BiDirIIOPServiceContextHelper.write(out, biDirCtxt);

            //
            // Fill in the bidir service context
            // 
            org.omg.IOP.ServiceContext context = 
                new org.omg.IOP.ServiceContext();
            context.context_id = org.omg.IOP.BI_DIR_IIOP.value;
            context.context_data = buf.data();

            //
            // Create and fill the return context list
            // 
            scl = new org.omg.IOP.ServiceContext[1];
            scl[0] = context;
            return scl;
        }

        //
        // we don't have a bidir service context so return an array of
        // length 0
        // 
        scl = new org.omg.IOP.ServiceContext[0];
	return scl;
    }

    public void
    handle_service_contexts(org.omg.IOP.ServiceContext[] contexts)
    {
        for(int i = 0; i < contexts.length; i++)
        {
            if(contexts[i].context_id == org.omg.IOP.BI_DIR_IIOP.value)
            {
                byte[] pOct = contexts[i].context_data;
                int len = contexts[i].context_data.length;

                com.ooc.OCI.Buffer buf = 
                    new com.ooc.OCI.Buffer(pOct, len);
                com.ooc.CORBA.InputStream in = 
                    new com.ooc.CORBA.InputStream(buf, 0, false);
                in._OB_readEndian();

                // 
                // unmarshal the octets back to the bidir format
                //
                org.omg.IIOP.BiDirIIOPServiceContext biDirCtxt = 
                    org.omg.IIOP.BiDirIIOPServiceContextHelper.read(in);

                // 
                // save the listening points in the transport
                // 
                _OB_setListenPoints(biDirCtxt.listen_points);
                
                break;
            }
        }
    }

    public synchronized boolean
    received_bidir_SCL()
    {
        if(listenPoints_ == null)
            return false;
        
        return (listenPoints_.length > 0);
    }

    public synchronized boolean
    endpoint_alias_match(com.ooc.OCI.ConnectorInfo connInfo)
    {
        //
        // we only deal with Connectors that are of our specific type,
        // namely IIOP connectors (and ConnectorInfos)
        //
        com.ooc.OCI.IIOP.ConnectorInfo_impl infoImpl;
        try
        {
            infoImpl = (com.ooc.OCI.IIOP.ConnectorInfo_impl)connInfo;
        }
        catch(ClassCastException ex)
        {
            return false;
        }

        //
        // compare the endpoint information in this connector with the
        // various endpoint inforamtion in our listenMap_
        //
        if(listenPoints_ == null)
            return false;

        short port = infoImpl.remote_port();
        String host = infoImpl.remote_addr();
        
        for(int i = 0; i < listenPoints_.length; i++)
        {
            if((listenPoints_[i].port == port) && 
                com.ooc.OB.Net.CompareHosts(listenPoints_[i].host, host))
                return true;
        }
        
        return false;
    }

    public synchronized org.omg.IIOP.ListenPoint[]
    _OB_getListenPoints()
    {
        return listenPoints_;
    }

    public synchronized void
    _OB_setListenPoints(org.omg.IIOP.ListenPoint[] lp)
    {
        listenPoints_ = lp;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    TransportInfo_impl(Transport_impl transport,
		       com.ooc.OCI.Connector connector,
                       ListenerMap lm)
    {
        transport_ = transport;
        connectorInfo_ = connector.get_info();
        listenMap_ = lm;
    }

    TransportInfo_impl(Transport_impl transport,
		       com.ooc.OCI.Acceptor acceptor,
                       ListenerMap lm)
    {
        transport_ = transport;
        acceptorInfo_ = acceptor.get_info();
        listenMap_ = lm;
    }

    synchronized void
    _OB_callCloseCB(com.ooc.OCI.TransportInfo info)
    {
        int length = closeCBVec_.size();
        for(int i = 0 ; i < length ; i++)
        {
            com.ooc.OCI.CloseCB cb =
                (com.ooc.OCI.CloseCB)closeCBVec_.elementAt(i);
            cb.close_cb(info);
        }
    }

    synchronized void
    _OB_destroy()
    {
	transport_ = null;
    }
}
