// **********************************************************************
//
// Copyright (c) 2005-2006
// IONA Technologies PLC
// Dublin, Ireland and Waltham, MA, USA
//
// All Rights Reserved
//
// Licensed to apache under Apache 2.0 License - 2006
// **********************************************************************

package com.ooc.OCI.IIOP;

final public class Transport_impl 
    extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.Transport
{
    // This data member must not be private because the info object
    // must be able to access it
    public  java.net.Socket socket_;    // The socket
    private java.io.InputStream in_;    // The socket's input stream
    private java.io.OutputStream out_;  // The socket's output stream
    private boolean shutdown_;          // True if shutdown() was called
    private int soTimeout_ = 0;         // The value for setSoTimeout()
    private TransportInfo_impl info_;   // Transport information

    // ------------------------------------------------------------------
    // Private and protected member implementations
    // ------------------------------------------------------------------

    private void
    setSoTimeout(int t)
    {
	if(soTimeout_ != t)
	{
	    soTimeout_ = t;

	    try
	    {
		socket_.setSoTimeout(soTimeout_);
	    }
	    catch(java.net.SocketException ex)
	    {
		throw new org.omg.CORBA.COMM_FAILURE(
		    com.ooc.OB.MinorCodes.describeCommFailure(
			com.ooc.OB.MinorCodes.MinorSetSoTimeout) +
		    ": socket error during setSoTimeout: " +
		    ex.toString(), com.ooc.OB.MinorCodes.MinorSetSoTimeout,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);  
	    }
	    catch(java.lang.NullPointerException ex)
	    {
		throw new org.omg.CORBA.COMM_FAILURE(
		    com.ooc.OB.MinorCodes.describeCommFailure(
			com.ooc.OB.MinorCodes.MinorSetSoTimeout) +
		    ": NullPointerException error during setSoTimeout: " +
		    ex.toString(), com.ooc.OB.MinorCodes.MinorSetSoTimeout,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);  
	    }
	}
    }

    private void
    setBlock(boolean block)
    {
	if(block)
	    setSoTimeout(0);
	else
	    setSoTimeout(1);
    }

    //
    // Shutdown the sending or receiving side of a socket. If how == 0,
    // shutdown the receiving side. If how == 1, shutdown the sending
    // side. If how == 2, shutdown both.
    //
    private static void
    shutdownSocket(java.net.Socket socket, int how)
    {
        if(socket == null) // Socket already closed
            return;

        if(how == 2)
	{
            shutdownSocket(socket, 0);
	    shutdownSocket(socket, 1);
	    return;
	}

	try
	{
	    if(how == 0)
	    {
		socket.shutdownInput();
	    }
	    else if(how == 1)
	    {
		socket.shutdownOutput();
	    }
	    else
	    {
		throw new InternalError();
	    }
	}
        catch(java.net.SocketException ex)
        {
            //
            // Some VMs (namely JRockit) raise a SocketException if
            // the socket has already been closed.
	    // This exception can be ignored.
            //
        }
	catch(java.io.IOException ex)
	{
	    throw new InternalError();
	}
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return PLUGIN_ID.value;
    }

    public int
    tag()
    {
	return org.omg.IOP.TAG_INTERNET_IOP.value;
    }

    public com.ooc.OCI.SendReceiveMode
    mode()
    {
        return com.ooc.OCI.SendReceiveMode.SendReceive;
    }

    public int
    handle()
    {
	throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    public void
    close()
    {
	if(socket_ == null) // shutdown() may call close()
	    return;

	//
	// Call callbacks
	//
	info_._OB_callCloseCB(info_);

	//
	// Destroy the info object
	//
	info_._OB_destroy();

	//
	// I must set socket_ to null *before* the close or the code
	// below, to avoid a race condition with send/receive
	//

	//
	// Close the socket
	//
	java.net.Socket saveSocket = socket_;
	socket_ = null; // Must be set to null before the shutdown/close
        shutdownSocket(saveSocket, 2); // This helps to unblock threads
                                       // blocking in recv()
        try
        {
            saveSocket.close();
        }
        catch(java.io.IOException ex)
        {
        }
    }

    public void
    shutdown()
    {
        shutdownSocket(socket_, 1); // Shutdown send side only
    }

    public void
    receive(com.ooc.OCI.Buffer buf, boolean block)
    {
	setBlock(block);

	while(!buf.is_full())
	{
	    try
	    {
		int result = in_.read(buf.data(), buf.pos(),
                                      buf.rest_length());
		if(result <= 0)
		{
		    throw new org.omg.CORBA.COMM_FAILURE(
                        com.ooc.OB.MinorCodes.describeCommFailure(
                            com.ooc.OB.MinorCodes.MinorRecvZero),
                        com.ooc.OB.MinorCodes.MinorRecvZero,
			org.omg.CORBA.CompletionStatus.COMPLETED_NO);
		}
		buf.advance(result);
	    }
	    catch(java.io.InterruptedIOException ex)
	    {
		buf.advance(ex.bytesTransferred);

		if(!block)
		    return;
	    }
	    catch(java.io.IOException ex)
	    {
		throw new org.omg.CORBA.COMM_FAILURE(
		    com.ooc.OB.MinorCodes.describeCommFailure(
			com.ooc.OB.MinorCodes.MinorRecv) +
                    ": I/O error during read: " + ex.toString(),
		    com.ooc.OB.MinorCodes.MinorRecv,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);  
	    }
	    catch(java.lang.NullPointerException ex)
	    {
		throw new org.omg.CORBA.COMM_FAILURE(
		    com.ooc.OB.MinorCodes.describeCommFailure(
			com.ooc.OB.MinorCodes.MinorRecv) +
                    ": NullPointerException during read",
		    com.ooc.OB.MinorCodes.MinorRecv,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);  
	    }
	}
    }

    public boolean
    receive_detect(com.ooc.OCI.Buffer buf, boolean block)
    {
	setBlock(block);
	
	while(!buf.is_full())
	{
	    try
	    {
		int result = in_.read(buf.data(), buf.pos(),
                                      buf.rest_length());
		if(result <= 0)
		    return false;
		buf.advance(result);
	    }
	    catch(java.io.InterruptedIOException ex)
	    {
		buf.advance(ex.bytesTransferred);

		if(!block)
		    return true;
	    }
	    catch(java.io.IOException ex)
	    {
		return false;
	    }
	    catch(java.lang.NullPointerException ex)
	    {
		return false;
	    }
	}

	return true;
    }

    public void
    receive_timeout(com.ooc.OCI.Buffer buf, int t)
    {
	if(t < 0)
	    throw new InternalError();

        if(t == 0)
        {
            receive(buf, false);
            return;
        }

	setSoTimeout(t);

	while(!buf.is_full())
	{
	    try
	    {
		int result = in_.read(buf.data(), buf.pos(),
                                      buf.rest_length());
		if(result <= 0)
		{
		    throw new org.omg.CORBA.COMM_FAILURE(
                        com.ooc.OB.MinorCodes.describeCommFailure(
                            com.ooc.OB.MinorCodes.MinorRecvZero),
                        com.ooc.OB.MinorCodes.MinorRecvZero,
			org.omg.CORBA.CompletionStatus.COMPLETED_NO);
		}
		buf.advance(result);
	    }
	    catch(java.io.InterruptedIOException ex)
	    {
		buf.advance(ex.bytesTransferred);
		return;
	    }
	    catch(java.io.IOException ex)
	    {
		throw new org.omg.CORBA.COMM_FAILURE(
		    com.ooc.OB.MinorCodes.describeCommFailure(
			com.ooc.OB.MinorCodes.MinorRecv) +
                    ": I/O error during read: " + ex.toString(),
		    com.ooc.OB.MinorCodes.MinorRecv,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);  
	    }
	    catch(java.lang.NullPointerException ex)
	    {
		throw new org.omg.CORBA.COMM_FAILURE(
		    com.ooc.OB.MinorCodes.describeCommFailure(
			com.ooc.OB.MinorCodes.MinorRecv) +
                    ": NullPointerException during read",
		    com.ooc.OB.MinorCodes.MinorRecv,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);  
	    }
	}
    }

    public boolean
    receive_timeout_detect(com.ooc.OCI.Buffer buf, int t)
    {
	if(t < 0)
	    throw new InternalError();

        if(t == 0)
            return receive_detect(buf, false);

	setSoTimeout(t);

	while(!buf.is_full())
	{
	    try
	    {
		int result = in_.read(buf.data(), buf.pos(),
                                      buf.rest_length());
		if(result <= 0)
                    return false;
		buf.advance(result);
	    }
	    catch(java.io.InterruptedIOException ex)
	    {
		buf.advance(ex.bytesTransferred);
                return true;
	    }
	    catch(java.io.IOException ex)
	    {
                return false;
	    }
	    catch(java.lang.NullPointerException ex)
	    {
                return false;
	    }
	}

        return true;
    }

    public void
    send(com.ooc.OCI.Buffer buf, boolean block)
    {
	setBlock(block);
	
	while(!buf.is_full())
	{
	    try
	    {
		out_.write(buf.data(), buf.pos(), buf.rest_length());
		out_.flush();
		buf.pos(buf.length());
	    }
	    catch(java.io.InterruptedIOException ex)
	    {
		buf.advance(ex.bytesTransferred);

		if(!block)
		    return;
	    }
	    catch(java.io.IOException ex)
	    {
		throw new org.omg.CORBA.COMM_FAILURE(
		    com.ooc.OB.MinorCodes.describeCommFailure(
			com.ooc.OB.MinorCodes.MinorSend) +
                    ": I/O error during write: " + ex.toString(),
		    com.ooc.OB.MinorCodes.MinorSend,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);  
	    }
	    catch(java.lang.NullPointerException ex)
	    {
		throw new org.omg.CORBA.COMM_FAILURE(
		    com.ooc.OB.MinorCodes.describeCommFailure(
			com.ooc.OB.MinorCodes.MinorSend) +
                    ": NullPointerException during write",
		    com.ooc.OB.MinorCodes.MinorSend,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);  
	    }
	}
    }

    public boolean
    send_detect(com.ooc.OCI.Buffer buf, boolean block)
    {
	setBlock(block);
	
	while(!buf.is_full())
	{
	    try
	    {
		out_.write(buf.data(), buf.pos(), buf.rest_length());
		out_.flush();
		buf.pos(buf.length());
	    }
	    catch(java.io.InterruptedIOException ex)
	    {
		buf.advance(ex.bytesTransferred);

		if(!block)
		    return true;
	    }
	    catch(java.io.IOException ex)
	    {
		return false;
	    }
	    catch(java.lang.NullPointerException ex)
	    {
		return false;
	    }
	}

	return true;
    }

    public void
    send_timeout(com.ooc.OCI.Buffer buf, int t)
    {
	if(t < 0)
	    throw new InternalError();

        if(t == 0)
        {
            send(buf, false);
            return;
        }

	setSoTimeout(t);
	
	while(!buf.is_full())
	{
	    try
	    {
		out_.write(buf.data(), buf.pos(), buf.rest_length());
		out_.flush();
		buf.pos(buf.length());
	    }
	    catch(java.io.InterruptedIOException ex)
	    {
		buf.advance(ex.bytesTransferred);
		return;
	    }
	    catch(java.io.IOException ex)
	    {
		throw new org.omg.CORBA.COMM_FAILURE(
		    com.ooc.OB.MinorCodes.describeCommFailure(
			com.ooc.OB.MinorCodes.MinorSend) +
                    ": I/O error during write: " + ex.toString(),
		    com.ooc.OB.MinorCodes.MinorSend,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);  
	    }
	    catch(java.lang.NullPointerException ex)
	    {
		throw new org.omg.CORBA.COMM_FAILURE(
		    com.ooc.OB.MinorCodes.describeCommFailure(
			com.ooc.OB.MinorCodes.MinorSend) +
                    ": NullPointerException during write",
		    com.ooc.OB.MinorCodes.MinorSend,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);  
	    }
	}
    }

    public boolean
    send_timeout_detect(com.ooc.OCI.Buffer buf, int t)
    {
        if(t < 0)
            throw new InternalError();

        if(t == 0)
            return send_detect(buf, false);

        setSoTimeout(t);
        
        while(!buf.is_full())
        {
            try
            {
                out_.write(buf.data(), buf.pos(), buf.rest_length());
                out_.flush();
                buf.pos(buf.length());
            }
            catch(java.io.InterruptedIOException ex)
            {
                buf.advance(ex.bytesTransferred);
                return true;
            }
            catch(java.io.IOException ex)
            {
                return false;
            }
            catch(java.lang.NullPointerException ex)
            {
                return false;
            }
        }

        return true;
    }

    public com.ooc.OCI.TransportInfo
    get_info()
    {
        return info_;
    }
 
    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    public
    Transport_impl(com.ooc.OCI.Connector connector,
		   java.net.Socket socket,
                   ListenerMap lm)
    {
	socket_ = socket;
        shutdown_ = false;

        //
        // Cache the streams associated with the socket, for
        // performance reasons
        //
        try
        {
            in_ = socket_.getInputStream();
            out_ = socket_.getOutputStream();
        }
        catch(java.io.IOException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSocket) +
                    ": unable to obtain socket InputStream: " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorSocket,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);  
        }

        //
        // Since the Constructor of TransportInfo uses this object create
        // it after all members are initialized
        //
        info_ = new TransportInfo_impl(this, connector, lm);
    }

    public
    Transport_impl(com.ooc.OCI.Acceptor acceptor,
		   java.net.Socket socket,
                   ListenerMap lm)
    {
	socket_ = socket;
        shutdown_ = false;

        //
        // Cache the streams associated with the socket, for
        // performance reasons
        //
        try
        {
            in_ = socket_.getInputStream();
            out_ = socket_.getOutputStream();
        }
        catch(java.io.IOException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSocket) +
                    ": unable to obtain socket InputStream: " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorSocket,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);  
        }

        //
        // Since the Constructor of TransportInfo uses this object create
        // it after all members are initialized
        //
        info_ = new TransportInfo_impl(this, acceptor, lm);
    }

    public void
    finalize()
	throws Throwable
    {
	if(socket_ != null)
	    close();

	super.finalize();
    }
}
